'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _assign2 = require('lodash/assign');

var _assign3 = _interopRequireDefault(_assign2);

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _jquery = require('jquery');

var _jquery2 = _interopRequireDefault(_jquery);

var _utils = require('../utils/utils');

var _utils2 = _interopRequireDefault(_utils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = {
    props: ['model', 'global', 'options'],
    data: function data() {
        return {
            dragger: {
                pageX: 0,
                pageY: 0,
                left: 0,
                top: 0,
                dx: 0,
                dy: 0
            }
        };
    },

    methods: {
        isDragable: function isDragable(e) {
            var model = this.model;

            if (!model || model.lock || model.$draging || !model.dragable || e.button !== 0 || this.options.mode !== 'design' && this.options.mode !== 'flow' || _utils2.default.isEditable(e.target)) {
                return false;
            }

            return true;
        },
        initDragger: function initDragger(element) {
            var self = this;
            var doc = (0, _jquery2.default)(document);
            var global = this.global;
            var dragger = this.dragger;

            var lockDirection = false;
            var handles = {
                init: function init(e, sourceEvent) {
                    e = sourceEvent || e;
                    if (!self.isDragable(e) || e.altKey) {
                        return;
                    }

                    var model = self.model;

                    (0, _assign3.default)(dragger, {
                        left: model.left,
                        top: model.top,
                        pageX: e.pageX,
                        pageY: e.pageY
                    });

                    doc.on('mousemove', handles.move);
                    doc.one('mouseup', handles.cancel);
                },
                move: function move(e) {
                    var model = self.model;
                    if (!model) {
                        return;
                    }

                    e.preventDefault();

                    var zoom = global.zoom;
                    dragger.dx = (e.pageX - dragger.pageX) / zoom;
                    dragger.dy = (e.pageY - dragger.pageY) / zoom;

                    if (e.shiftKey) {
                        if (!lockDirection) {
                            if (Math.abs(dragger.dx) >= 3 || Math.abs(dragger.dy) >= 3) {
                                var ratio = Math.abs(dragger.dx / dragger.dy);
                                lockDirection = ratio > 1 ? 'x' : 'y';
                            }
                        }
                        if (lockDirection === 'x') {
                            dragger.dy = 0;
                        } else {
                            dragger.dx = 0;
                        }
                    }

                    var dOffsetMin = 3;
                    if (!model.$draging && (Math.abs(dragger.dx) >= dOffsetMin || Math.abs(dragger.dy) >= dOffsetMin)) {
                        model.$draging = true;

                        self.$events.$emit('editor.dragger.active', model);
                    }

                    if (model.$draging) {
                        self.$events.$emit('editor.dragger.moving', dragger, model, e);
                    }
                },
                cancel: function cancel() {
                    doc.off('mousemove', handles.move);

                    var model = self.model;
                    if (model && model.$draging) {
                        model.$draging = false;

                        self.$events.$emit('editor.dragger.inactive', model);
                    }
                    lockDirection = false;
                }
            };

            element.on('mousedown.editor-dragger', handles.init);

            this.$on('destroy', function () {
                element.off('.editor-dragger');
            });
        },
        initScaler: function initScaler(element) {}
    },

    events: {
        'editor.dragger.moving': function editorDraggerMoving(drag, element) {
            if (element !== this.model || element.lock) {
                return false;
            }

            var dx = drag.dx,
                dy = drag.dy;
            var left = drag.left + drag.dx,
                top = drag.top + drag.dy;


            if (element.$dragLimit) {
                var rotateVector = _utils2.default.getVectorRotator(element.rotate);

                var _element$$getDragLimi = element.$getDragLimit(),
                    minLeft = _element$$getDragLimi.minLeft,
                    maxLeft = _element$$getDragLimi.maxLeft,
                    minTop = _element$$getDragLimi.minTop,
                    maxTop = _element$$getDragLimi.maxTop,
                    centerDeltaX = _element$$getDragLimi.centerDeltaX,
                    centerDeltaY = _element$$getDragLimi.centerDeltaY;

                var _rotateVector = rotateVector(dx, dy),
                    _rotateVector2 = _slicedToArray(_rotateVector, 2),
                    dx_ = _rotateVector2[0],
                    dy_ = _rotateVector2[1];

                var _rotateVector3 = rotateVector(centerDeltaX, centerDeltaY),
                    _rotateVector4 = _slicedToArray(_rotateVector3, 2),
                    centerDeltaX_ = _rotateVector4[0],
                    centerDeltaY_ = _rotateVector4[1];

                var clampedDeltaX_ = _utils2.default.clamp(centerDeltaX_ + dx_, minLeft, maxLeft);
                var clampedDeltaY_ = _utils2.default.clamp(centerDeltaY_ + dy_, minTop, maxTop);

                var _rotateVector5 = rotateVector(clampedDeltaX_, clampedDeltaY_, true),
                    _rotateVector6 = _slicedToArray(_rotateVector5, 2),
                    clampedDeltaX = _rotateVector6[0],
                    clampedDeltaY = _rotateVector6[1];

                dx = clampedDeltaX - centerDeltaX;
                dy = clampedDeltaY - centerDeltaY;
                left = drag.left + dx;
                top = drag.top + dy;
            }

            if (element.$customDragMove) {
                this.$events.$emit('editor.drag.customMoving', { element: element, dx: dx, dy: dy });
            } else {
                element.left = left;
                element.top = top;
            }
        }
    },

    mounted: function mounted() {
        var _this = this;

        this.$nextTick(function () {
            var element = (0, _jquery2.default)(_this.$el);
            if (!element.length) {
                return;
            }

            _this.initDragger(element);
            _this.initScaler(element);
        });
    },
    beforeDestroy: function beforeDestroy() {
        this.$emit('destroy');
    }
};
module.exports = exports['default'];