'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _cloneDeep2 = require('lodash/cloneDeep');

var _cloneDeep3 = _interopRequireDefault(_cloneDeep2);

var _merge2 = require('lodash/merge');

var _merge3 = _interopRequireDefault(_merge2);

var _isEmpty2 = require('lodash/isEmpty');

var _isEmpty3 = _interopRequireDefault(_isEmpty2);

var _forEach2 = require('lodash/forEach');

var _forEach3 = _interopRequireDefault(_forEach2);

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _jquery = require('jquery');

var _jquery2 = _interopRequireDefault(_jquery);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _utils = require('../utils/utils');

var _utils2 = _interopRequireDefault(_utils);

var _vueInherit = require('../utils/vue-inherit');

var _vueInherit2 = _interopRequireDefault(_vueInherit);

var _editorElement = require('./editor-element');

var _editorElement2 = _interopRequireDefault(_editorElement);

var _editorElementSvg = require('./editor-element-svg.html');

var _editorElementSvg2 = _interopRequireDefault(_editorElementSvg);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = (0, _vueInherit2.default)(_editorElement2.default, {
    template: _editorElementSvg2.default,
    data: function data() {
        return {
            svg: null,
            isContainer: false,
            viewBox: [null, null, null, null],
            boxes: {
                container: null,
                nw: null,
                n: null,
                ne: null,
                w: null,
                c: null,
                e: null,
                sw: null,
                s: null,
                se: null
            },
            errorMessage: ''
        };
    },

    computed: {
        supportTransform: function supportTransform() {
            return !!this.model.containerTransform;
        },
        cssStyle: function cssStyle() {
            var rect = this.rect;
            var padding = rect.padding;


            return {
                height: rect.height + padding[0] + padding[2] + 'px',
                width: rect.width + padding[1] + padding[3] + 'px',
                transform: this.transform.toString(),
                left: rect.left + 'px',
                top: rect.top + 'px',
                opacity: this.opacity
            };
        }
    },
    methods: {
        load: function load() {
            var model = this.model;
            var content = model.content,
                url = model.url;


            if (content) {
                return _bluebird2.default.try(function () {
                    return _jquery2.default.parseXML(content);
                });
            }

            return _utils2.default.loadXML(url);
        },
        buildSvg: function buildSvg(xml) {
            xml = (0, _jquery2.default)(xml);
            this.fixSvgAttrs(xml);

            var html = this.serializeXML(xml[0]);

            html = html.replace(/(fill|stroke)="{{(.*?)}}"/g, ':$1="$2"');

            this.model.$content = html;

            this.errorMessage = '';
            this.loading = false;

            var panel = (0, _jquery2.default)('.element-main', this.$el);
            panel.html(html);

            var Vue = this.constructor;
            while (Vue.super) {
                Vue = Vue.super;
            }

            this.viewBox = ((0, _jquery2.default)(html).filter('svg').attr('viewBox') || (0, _jquery2.default)(html).find('svg').attr('viewBox') || '0 0 0 0').split(' ').map(function (x) {
                return parseInt(x, 10);
            });

            if (this.isContainer) {
                this.boxes = this.getBBoxes();
            }

            var elementVM = this;

            var svgVM = {
                el: panel[0],
                data: elementVM.model,
                methods: {
                    getComputedTransform: function getComputedTransform(dir) {
                        if (!this.containerTransform) {
                            return null;
                        }

                        var _containerTransform$d = this.containerTransform[dir],
                            sx = _containerTransform$d.sx,
                            sy = _containerTransform$d.sy,
                            tx = _containerTransform$d.tx,
                            ty = _containerTransform$d.ty;

                        var box = elementVM.boxes[dir];

                        if (!box) {
                            return null;
                        }
                        var mx = box.x + box.width / 2,
                            my = box.y + box.height / 2;

                        return 'translate(' + tx + ' ' + ty + ') translate(' + mx + ' ' + my + ') scale(' + sx + ' ' + sy + ') translate(' + -mx + ' ' + -my + ')';
                    }
                }
            };

            if (this.isContainer) {
                svgVM.computed = {
                    viewBox: function viewBox() {
                        var _elementVM$viewBox = _slicedToArray(elementVM.viewBox, 2),
                            x1 = _elementVM$viewBox[0],
                            y1 = _elementVM$viewBox[1];

                        return x1 + ' ' + y1 + ' ' + (x1 + this.width) + ' ' + (y1 + this.height);
                    },
                    tScale: function tScale() {
                        if (!this.containerTransform) {
                            return null;
                        }

                        return 'scale(' + this.containerTransform.scale + ')';
                    },
                    tNW: function tNW() {
                        return this.getComputedTransform('nw');
                    },
                    tN: function tN() {
                        return this.getComputedTransform('n');
                    },
                    tNE: function tNE() {
                        return this.getComputedTransform('ne');
                    },
                    tW: function tW() {
                        return this.getComputedTransform('w');
                    },
                    tC: function tC() {
                        return this.getComputedTransform('c');
                    },
                    tE: function tE() {
                        return this.getComputedTransform('e');
                    },
                    tS: function tS() {
                        return this.getComputedTransform('s');
                    },
                    tSE: function tSE() {
                        return this.getComputedTransform('se');
                    },
                    tSW: function tSW() {
                        return this.getComputedTransform('sw');
                    }
                };
            }

            elementVM.svg = new Vue(svgVM);

            elementVM.$on('destroy', function () {
                if (elementVM.svg) {
                    elementVM.svg.$destroy(true, true);
                }
            });
        },
        fixSvgAttrs: function fixSvgAttrs(xml) {
            var _this = this;

            var model = this.model;

            var customColorClass = '.editor-props-color';

            var propParsers = {
                xmlns: String,
                preserveAspectRatio: String,
                viewBox: String,
                x: parseInt,
                y: parseInt
            };

            var uid = 0;
            var colors = {};
            var colorsMap = {};
            var addCustomColor = function addCustomColor(node, name) {
                var val = node.getAttribute(name);
                var k = colorsMap[val];

                if (!k) {
                    k = colorsMap[val] = 'color_p_' + ++uid;

                    colors[k] = val;
                }

                node.setAttribute(':' + name, 'colors.' + k);
            };

            xml.find('> svg').each(function (i, node) {
                node = (0, _jquery2.default)(node);

                var attributes = [].slice.call(node[0].attributes);

                (0, _forEach3.default)(attributes, function (item) {
                    var prop = item.name;
                    if (propParsers[prop]) {
                        var parser = propParsers[prop];
                        var val = parser(node.attr(prop));
                        node.attr(prop, val);
                    } else {
                        node.removeAttr(prop);
                    }
                });

                node.find(customColorClass).each(function (i, node) {
                    if (node.hasAttribute('fill')) {
                        addCustomColor(node, 'fill');
                    }

                    if (node.hasAttribute('stroke')) {
                        addCustomColor(node, 'stroke');
                    }
                });

                node.attr(':width', 'width');
                node.attr(':height', 'height');

                _this.isContainer = !!node.has('.editor-svg-container').length;
                if (_this.isContainer) {
                    node.attr(':view-box.camel', 'viewBox');

                    node.find('.editor-svg-container').each(function (i, container) {
                        (0, _jquery2.default)(container).attr(':transform', 'tScale');
                    });

                    var dirs = ['nw', 'n', 'ne', 'w', 'c', 'e', 'sw', 's', 'se'];

                    dirs.forEach(function (dir) {
                        var computedName = 't' + dir.toUpperCase();
                        node.find('.editor-svg-' + dir + ' *').each(function (i, grid) {
                            (0, _jquery2.default)(grid).attr(':transform', computedName);
                        });
                    });
                }
            });

            if (!(0, _isEmpty3.default)(colors)) {
                model.colors = (0, _merge3.default)(colors, model.colors);
            }
        },
        getBBox: function getBBox(name) {
            return this.$el.getElementsByClassName('editor-svg-' + name)[0].getBBox();
        },
        getContainerBBox: function getContainerBBox() {
            return {
                width: this.model.width,
                height: this.model.height,
                x: this.left,
                y: this.top
            };
        },
        getBBoxes: function getBBoxes() {
            var getBBox = this.getBBox,
                getContainerBBox = this.getContainerBBox;


            return {
                container: getContainerBBox(),
                se: getBBox('se'),
                s: getBBox('s'),
                sw: getBBox('sw'),
                e: getBBox('e'),
                c: getBBox('c'),
                w: getBBox('w'),
                ne: getBBox('ne'),
                n: getBBox('n'),
                nw: getBBox('nw')
            };
        },
        setTransform: function setTransform(dir, key, value) {
            var containerTransform = this.model.containerTransform;

            if (!containerTransform || !containerTransform[dir]) {
                return;
            }
            containerTransform[dir][key] = value;
        },
        resizeInit: function resizeInit(dir) {
            if (!this.isContainer) {
                return;
            }

            this.$svgResizeMeta = {
                dir: dir,
                originalBoxes: this.boxes,
                currentBoxes: this.getBBoxes(),
                containerTransform: (0, _cloneDeep3.default)(this.model.containerTransform),
                width: this.model.width,
                height: this.model.height
            };
        },
        resizeEnd: function resizeEnd() {
            delete this.$svgResizeMeta;
        },
        setScale: function setScale(value) {
            if (!this.model.containerTransform) {
                return;
            }

            this.model.containerTransform.scale = value;
        },
        setSize: function setSize(_ref) {
            var dir = _ref.dir,
                dx = _ref.dx,
                dy = _ref.dy;
            var $svgResizeMeta = this.$svgResizeMeta,
                setTransform = this.setTransform,
                setScale = this.setScale;
            var _model = this.model,
                $minWidth = _model.$minWidth,
                $minHeight = _model.$minHeight;


            if (!$svgResizeMeta) {
                return;
            }

            var containerTransform = $svgResizeMeta.containerTransform,
                originalBoxes = $svgResizeMeta.originalBoxes,
                currentBoxes = $svgResizeMeta.currentBoxes;


            if (!containerTransform) {
                return;
            }

            var currentTransform = containerTransform;

            var currentScale = currentTransform.scale;

            if ($minWidth) {
                dx = Math.max($minWidth - currentBoxes.container.width, dx);
            }
            if ($minHeight) {
                dy = Math.max($minHeight - currentBoxes.container.height, dy);
            }

            if (dir === 'w' || dir === 'e') {
                var middleGroup = ['n', 'c', 's'];
                middleGroup.forEach(function (dir) {
                    var baseTx = currentTransform[dir].tx;
                    var originalWidth = originalBoxes[dir].width;
                    var baseWidth = currentBoxes[dir].width;

                    setTransform(dir, 'sx', (baseWidth + dx / currentScale) / originalWidth);
                    setTransform(dir, 'tx', baseTx + dx / currentScale / 2);
                });

                var rightGroup = ['ne', 'e', 'se'];
                rightGroup.forEach(function (dir) {
                    var baseTx = currentTransform[dir].tx;
                    setTransform(dir, 'tx', baseTx + dx / currentScale);
                });
            } else if (dir === 'n' || dir === 's') {
                    var _middleGroup = ['w', 'c', 'e'];
                    _middleGroup.forEach(function (dir) {
                        var baseTy = currentTransform[dir].ty;
                        var originalHeight = originalBoxes[dir].height;
                        var baseHeight = currentBoxes[dir].height;

                        setTransform(dir, 'sy', (baseHeight + dy / currentScale) / originalHeight);
                        setTransform(dir, 'ty', baseTy + dy / currentScale / 2);
                    });

                    var bottomGroup = ['sw', 's', 'se'];
                    bottomGroup.forEach(function (dir) {
                        var baseTy = currentTransform[dir].ty;
                        setTransform(dir, 'ty', baseTy + dy / currentScale);
                    });
                } else if (dir.length === 2) {
                        var currentWidth = currentBoxes.container.width;
                        var scale = (currentWidth + dx) / (currentWidth / currentTransform.scale);
                        setScale(scale);
                    }
        },
        serializeXML: function serializeXML(xml) {
            var str = '';

            if (window.XMLSerializer) {
                str = new XMLSerializer().serializeToString(xml);
            } else {
                str = xml.xml;
            }

            return str;
        },
        render: function render() {
            var _this2 = this;

            this.load().then(function (xml) {
                _this2.buildSvg(xml);
            }).catch(function (err) {
                _this2.errorMessage = err.message;
            });
        }
    },
    events: {
        'editor.groupContent.scale': function editorGroupContentScale(model, ratio) {
            var _this3 = this;

            var parentModel = this.$parent.model;
            if (parentModel !== model) {
                return;
            }

            var _model2 = this.model,
                width = _model2.width,
                height = _model2.height;

            this.$scaling = true;
            this.resizeInit('se');
            this.setSize({
                dir: 'se',
                dx: width * (ratio - 1),
                dy: height * (ratio - 1)
            });
            this.resizeEnd();

            setTimeout(function () {
                delete _this3.$scaling;
            }, 0);
        },
        'editor.transform.active': function editorTransformActive(model, data) {
            if (!this.supportTransform) {
                return;
            }

            if (!(model === this.model) && !(model.type === 'group' && model.elements.indexOf(this.model) > -1)) {
                return;
            }

            var _getBBoxes = this.getBBoxes(),
                container = _getBBoxes.container,
                n = _getBBoxes.n,
                w = _getBBoxes.w,
                nw = _getBBoxes.nw;

            var originalContainer = this.boxes.container;
            var scale = this.model.containerTransform.scale;

            if (data.dir.length === 1) {
                this.model.$minWidth = container.width - n.width * scale;
                this.model.$minHeight = container.height - w.height * scale;
            } else if (data.dir.length === 2) {
                var minScale = 10 / nw.width;
                var baseMinWidth = originalContainer.width * minScale;
                this.model.$minWidth = baseMinWidth;
                this.model.$minHeight = baseMinWidth * (container.height / container.width);
            }

            this.resizeInit(data.dir);
        },
        'editor.transform.resizing': function editorTransformResizing(drag) {
            this.setSize(drag);
        },
        'editor.transform.inactive': function editorTransformInactive() {
            this.resizeEnd();
            delete this.model.$minWidth;
            delete this.model.$minHeight;
        }
    },
    watch: {
        'model.content': function modelContent() {
            var _this4 = this;

            this.checkLoad().then(function () {
                _this4.render();
            });
        },
        'model.url': function modelUrl() {
            var _this5 = this;

            this.checkLoad().then(function () {
                _this5.render();
            });
        },
        'model.width': function modelWidth(width, prevWidth) {
            if (!this.supportTransform || this.$svgResizeMeta || this.$scaling) {
                return;
            }

            this.resizeInit('e');
            this.setSize({
                dir: 'e',
                dx: width - prevWidth,
                dy: 0
            });
            this.resizeEnd();
        },
        'model.height': function modelHeight(height, prevHeight) {
            if (!this.supportTransform || this.$svgResizeMeta || this.$scaling) {
                return;
            }

            this.resizeInit('s');
            this.setSize({
                dir: 's',
                dx: 0,
                dy: height - prevHeight
            });
            this.resizeEnd();
        }
    },
    mounted: function mounted() {
        this.render();
    }
});
module.exports = exports['default'];