'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _uniq2 = require('lodash/uniq');

var _uniq3 = _interopRequireDefault(_uniq2);

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _jquery = require('jquery');

var _jquery2 = _interopRequireDefault(_jquery);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _loader = require('../utils/loader');

var _loader2 = _interopRequireDefault(_loader);

var _vueInherit = require('../utils/vue-inherit');

var _vueInherit2 = _interopRequireDefault(_vueInherit);

var _utils = require('../utils/utils');

var _editorElement = require('./editor-element');

var _editorElement2 = _interopRequireDefault(_editorElement);

var _editorElementText = require('./editor-element-text.html');

var _editorElementText2 = _interopRequireDefault(_editorElementText);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = (0, _vueInherit2.default)(_editorElement2.default, {
    template: _editorElementText2.default,
    data: function data() {
        return {
            fallbackFonts: 'Arial',
            minFontSize: 15,
            lastFont: null
        };
    },

    computed: {
        hasEffects: function hasEffects() {
            return this.model.textEffects && this.effectedTextEffects.length;
        },
        resize: function resize() {
            return this.model.resize;
        },
        isVertical: function isVertical() {
            var writingMode = this.model.writingMode;

            return writingMode && writingMode.indexOf('vertical') > -1;
        },
        contentsHTML: function contentsHTML() {
            var _this = this;

            var contents = this.model.contents.map(function (item) {
                if (item && item.fontFamily) {
                    var font = _this.getCloseFont(item.fontFamily);

                    return Object.assign({}, item, font ? {
                        fontFamily: font.family || font.name
                    } : null);
                }

                return item;
            });
            var html = _utils.serialize.fromHTML(contents, this.global.zoom);
            return html;
        },
        fontFamily: function fontFamily() {
            var font = this.lastFont;
            if (!font || this.model.$loaded) {
                font = this.getCloseFont();
            }

            var fallbackFonts = this.fallbackFonts;
            var family = font ? font.family || font.name : '';

            return '"' + family + '"' + (fallbackFonts ? ',' + fallbackFonts : '');
        },
        color: function color() {
            return _utils.onecolor.cssa(this.model.color);
        },
        backgroundColor: function backgroundColor() {
            return _utils.onecolor.cssa(this.model.backgroundColor);
        },
        fontSize: function fontSize() {
            return this.global.zoom * this.model.fontSize;
        },
        fontSizeScale: function fontSizeScale() {
            return Math.min(1, this.fontSize / this.minFontSize);
        },
        letterSpacingScale: function letterSpacingScale() {
            return this.fontSizeScale < 1 ? this.minFontSize / this.model.fontSize : this.global.zoom;
        },
        transformOrigin: function transformOrigin() {
            var origin = [0, 0];
            var align = this.model.verticalAlign;

            if (!this.isVertical) {
                origin[0] = 0;
                origin[1] = { top: 0, middle: '50%', bottom: '100%' }[align];
            } else {
                    origin[0] = { top: '100%', middle: '50%', bottom: 0 }[align];
                    origin[1] = 0;
                }

            return origin.join(' ');
        },
        mainTransform: function mainTransform() {
            var fontSizeScale = this.fontSizeScale;

            return fontSizeScale < 1 ? 'scale(' + fontSizeScale + ')' : null;
        },
        mainMinHeight: function mainMinHeight() {
            if (!this.isVertical) {
                var model = this.model;
                return this.fontSize * model.lineHeight * this.fontSizeScale + 'px';
            } else {
                return 'initial';
            }
        },
        mainMinWidth: function mainMinWidth() {
            if (this.isVertical) {
                var model = this.model;
                return this.fontSize * model.lineHeight * this.fontSizeScale + 'px';
            } else {
                return 'initial';
            }
        },
        mainWidth: function mainWidth() {
            var fontSizeScale = this.fontSizeScale;
            var width = 100 / fontSizeScale;

            if (this.isVertical) {
                return 'auto';
            } else {
                return width + '%';
            }
        },
        mainHeight: function mainHeight() {
            var fontSizeScale = this.fontSizeScale;
            var height = 100 / fontSizeScale;

            if (!this.isVertical) {
                return 'auto';
            } else {
                return height + '%';
            }
        },
        textShadow: function textShadow() {
            var data = this.model.textShadow;

            if (!data) {
                return '';
            }

            return [data.offsetX + 'px', data.offsetY + 'px', data.blurRadius + 'px', data.color].join(' ');
        },
        cssStyle: function cssStyle() {
            var rect = this.rect,
                model = this.model;


            return {
                height: rect.height + 'px',
                width: rect.width + 'px',
                transform: this.transform.toString(),
                left: rect.left + 'px',
                top: rect.top + 'px',
                opacity: this.opacity,
                overflow: model.resize === 7 ? 'hidden' : 'visible'
            };
        },
        textStyle: function textStyle() {
            return {
                fontSize: (this.fontSize >= this.minFontSize ? this.fontSize : this.minFontSize) + 'px',
                letterSpacing: this.letterSpacingScale * this.model.letterSpacing + 'px',

                verticalAlign: this.model.verticalAlign,
                transform: this.mainTransform,
                transformOrigin: this.transformOrigin,
                minWidth: this.mainMinWdith,
                minHeight: this.mainMinHeight,
                width: this.mainWidth,
                height: this.mainHeight
            };
        }
    },
    methods: {
        load: function load(name) {
            var _this2 = this;

            var options = this.options;
            var _model = this.model,
                contents = _model.contents,
                $blinking = _model.$blinking,
                fontFamily = _model.fontFamily;


            var names = null;
            if (name) {
                names = [name];
            } else {
                names = contents.filter(function (item) {
                    return item && item.fontFamily;
                }).map(function (item) {
                    return item.fontFamily;
                });

                names.push(fontFamily);

                names = (0, _uniq3.default)(names);
            }

            var fontLoads = names.map(function (name) {
                var font = _this2.getCloseFont(name);
                if (!font) {
                    return _bluebird2.default.resolve();
                }

                return _loader2.default.loadFont(_extends({}, font, {
                    display: $blinking && _this2.isShowBlink ? 'swap' : 'auto'
                }), options.fontLoadTimeout);
            });

            return _bluebird2.default.all(fontLoads);
        },
        getCloseFont: function getCloseFont() {
            var name = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
            var _options = this.options,
                fontList = _options.fontList,
                fontsMap = _options.fontsMap;


            if (!name) {
                name = this.model.fontFamily;
            }

            return fontsMap[name] || fontList[0];
        },
        getRect: function getRect() {
            var superMethods = _editorElement2.default[_editorElement2.default.length - 1].methods;
            var rect = superMethods.getRect.call(this);

            var zoom = this.global.zoom;

            var fontSizeScale = this.fontSizeScale;
            rect.height = rect.height * fontSizeScale + (this.model.padding[0] + this.model.padding[2]) * zoom;
            rect.width = rect.width * fontSizeScale + (this.model.padding[1] + this.model.padding[3]) * zoom;

            return rect;
        },
        resizeModel: function resizeModel(drag) {
            if (!this.model.autoScale && this.model.resize !== 4) return;

            var element = (0, _jquery2.default)(this.$el);
            var innerElement = element.find('.element-main');
            var zoom = this.global.zoom;
            if (innerElement.length) {
                element = innerElement;
            }

            var currHeight = (element.prop('offsetHeight') || 0) / zoom;
            var currWidth = (element.prop('offsetWidth') || 0) / zoom;

            var isHorizontal = this.model.writingMode === 'horizontal-tb';
            var dir = drag.dir;

            if (isHorizontal && (dir === 'w' || dir === 'e') && currHeight !== this.model.height) {
                this.model.height = currHeight;
            } else if (!isHorizontal && (dir === 'n' || dir === 's') && currWidth !== this.model.width) {
                this.model.width = currWidth;
            }
        },
        syncRect: function syncRect() {
            if (this.options.mode === 'preview') {
                return;
            }

            if (!this.model.autoScale && this.resize !== 4) {
                return;
            }

            clearTimeout(this.syncRectTimer);
            this.syncRectTimer = setTimeout(this.updateRect, 0);
        }
    },

    watch: {},
    events: {
        'editor.element.loaded': function editorElementLoaded(model) {
            if (model === this.model) {
                this.syncRect();
            }
        },
        'editor.transform.active': function editorTransformActive(drag, _ref) {
            var _this3 = this;

            var action = _ref.action;

            if (action !== 'resize') return;

            var _rect = this.rect,
                width = _rect.width,
                height = _rect.height;

            this.$textResizeMeta = {
                width: width,
                height: height,
                fontSize: this.model.fontSize,
                contents: this.model.contents.map(function (content) {
                    return parseFloat(content.fontSize || _this3.model.fontSize);
                })
            };
        },
        'editor.transform.inactive': function editorTransformInactive(model, drag, _ref2) {
            var action = _ref2.action;

            if (action !== 'resize') return;
            delete this.$textResizeMeta;
        },
        'editor.transform.resizing': function editorTransformResizing(drag, model) {
            if (model !== this.model) return;

            this.resizeModel(drag);

            if (!model.autoScale) {
                this.syncRect();
                return;
            }

            var isHorizontal = this.model.writingMode === 'horizontal-tb';
            var dir = drag.dir;

            if (isHorizontal && (dir === 'n' || dir === 's') || !isHorizontal && (dir === 'w' || dir === 'e')) {
                return;
            }

            if (dir.length > 1) {
                var currRect = this.getRect();
                var baseText = this.$textResizeMeta;

                var ratio = isHorizontal ? currRect.width / (baseText.width || 1) : currRect.height / (baseText.height || 1);

                var fitFontSize = function fitFontSize(baseFontSize, ratio) {
                    var newFontSize = Math.max(baseFontSize * ratio, 0.01);
                    if (newFontSize < 6) {
                        return 6;
                    } else if (newFontSize % 1 !== 0) {
                        return Math.floor(newFontSize * 10) / 10;
                    }
                    return newFontSize;
                };

                model.fontSize = fitFontSize(baseText.fontSize, ratio);
                model.contents = model.contents.map(function (content, i) {
                    return _extends({}, content, {
                        fontSize: fitFontSize(baseText.contents[i], ratio)
                    });
                });
            } else {
                    this.syncRect();
                }
        }
    },
    created: function created() {
        var _this4 = this;

        this.$watch('model.fontFamily', function (a, b) {
            if (_this4.model.$loaded) {
                _this4.lastFont = _this4.getCloseFont(b);
            }

            _this4.checkLoad();
        });
    },
    mounted: function mounted() {
        this.syncRect();
    }
});
module.exports = exports['default'];