'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _jquery = require('jquery');

var _jquery2 = _interopRequireDefault(_jquery);

var _editorContextmenu = require('./editor-contextmenu.html');

var _editorContextmenu2 = _interopRequireDefault(_editorContextmenu);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = {
    template: _editorContextmenu2.default,

    data: function data() {
        return {
            isShow: false,

            left: 0,
            top: 0,
            width: 250,

            point: null,
            layerList: []
        };
    },


    computed: {
        editor: function editor() {
            return this.$parent;
        }
    },

    methods: {
        getHeight: function getHeight() {
            return this.$el && this.$el.offsetHeight || 0;
        },
        setHeight: function setHeight(val) {
            this.$el.style.height = val;
        },
        setPosition: function setPosition(e) {
            var editor = this.editor;
            var containerRect = editor.containerRect;

            var left = e.pageX - containerRect.left + containerRect.scrollLeft;
            var top = e.pageY - containerRect.top + containerRect.scrollTop;

            var maxLeft = containerRect.width - this.width + containerRect.scrollLeft;
            var maxTop = containerRect.height - this.getHeight() + containerRect.scrollTop;

            var edgeOffset = 5;

            maxLeft -= edgeOffset;
            maxTop -= edgeOffset;

            this.left = left > maxLeft ? left - this.width - 2 : left + 2;
            this.top = Math.min(maxTop, top);
        },
        hide: function hide() {
            this.isShow = false;

            this.$events.$emit('editor.highlight.element', false);
        },
        getLayerList: function getLayerList(point, layout) {
            if (!point) {
                return [];
            }

            return this.editor.getElementsByPoint(point.x, point.y, layout);
        },
        selectLayer: function selectLayer(element) {
            this.editor.focusElement(element);
            this.hide();
        },
        copyElement: function copyElement() {
            this.editor.copyElement();
            this.isShow = false;
        },
        cutElement: function cutElement() {
            this.editor.cutElement();
            this.isShow = false;
        },
        pasteElement: function pasteElement() {
            this.editor.pasteElement();
            this.isShow = false;
        },
        pasteFromMenu: function pasteFromMenu() {
            this.isShow = false;

            if (this.editor.clipboard.length) {
                this.editor.pasteElement();
                return;
            }

            var commandStr = navigator.platform.indexOf('Mac') > -1 ? '⌘' : 'Ctrl';

            setTimeout(function () {
                alert('\u7C98\u8D34\u5916\u90E8\u56FE\u7247\u8BF7\u4F7F\u7528\u952E\u76D8\uFF1A ' + commandStr + ' + V');
            }, 0);
        }
    },

    filters: {
        hotkey: function hotkey(key) {
            var commandStr = navigator.platform.indexOf('Mac') > -1 ? '⌘' : 'Ctrl';
            return key && key.replace('command', commandStr);
        }
    },

    events: {
        'editor.contextmenu': function editorContextmenu(e) {
            var _this = this;

            this.hide();

            this.point = this.editor.pointFromEvent(e);

            var layerList = this.getLayerList(this.point);

            if (this.editor.mode === 'flow' && layerList.length === 0) {
                var pointLayout = this.editor.getLayoutByPoint(this.point);
                if (this.editor.currentLayout !== pointLayout) {
                    this.editor.toggleLayout(pointLayout);
                    layerList = this.getLayerList(this.point);
                }
            }

            this.layerList = layerList;

            this.setHeight('auto');
            this.isShow = true;

            this.$nextTick(function () {
                _this.setPosition(e);
            });
        },
        'editor.mousedown': function editorMousedown(e) {
            if (!_jquery2.default.contains(this.$el, e.target) && this.$el !== e.target) {
                this.isShow = false;
            }
        },
        'editor.click': function editorClick(e) {
            if (_jquery2.default.contains(this.$el, e.target) || this.$el === e.target) {
                e.preventDefault();
            }
        }
    },

    mounted: function mounted() {
        this.editor.contextmenu = this;
    },
    beforeDestroy: function beforeDestroy() {}
};
module.exports = exports['default'];