'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _forEach2 = require('lodash/forEach');

var _forEach3 = _interopRequireDefault(_forEach2);

var _utils = require('../utils/utils');

var _utils2 = _interopRequireDefault(_utils);

var _editorGuider = require('./editor-guider.html');

var _editorGuider2 = _interopRequireDefault(_editorGuider);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var HIDE_OFFSET = -10;

exports.default = {
    template: _editorGuider2.default,
    props: ['global', 'layout'],
    data: function data() {
        return {
            offset: 2,
            model: null,
            left: HIDE_OFFSET,
            top: HIDE_OFFSET
        };
    },

    computed: {
        zoom: function zoom() {
            return this.global.zoom;
        },
        styleH: function styleH() {
            return {
                display: this.left >= 0 ? 'block' : 'none',
                left: this.zoom * this.left + 'px',
                top: this.layout.top * this.zoom + 'px',
                height: this.layout.height * this.zoom + 'px'
            };
        },
        styleV: function styleV() {
            return {
                display: this.top >= 0 ? 'block' : 'none',
                top: this.zoom * (this.top + this.layout.top) + 'px'
            };
        }
    },
    methods: {
        getPoints: function getPoints(bbox) {
            var height = bbox.height;
            var width = bbox.width;
            var left = bbox.left;
            var top = bbox.top;

            return {
                v: [top, top + height / 2, top + height],
                h: [left, left + width / 2, left + width]
            };
        },
        getGuidePoint: function getGuidePoint(boxA, boxB) {
            var self = this;
            var aPoints = this.getPoints(boxA);
            var bPoints = this.getPoints(boxB);

            var offset = 2 * this.offset;
            var ret = {
                left: HIDE_OFFSET,
                top: HIDE_OFFSET
            };

            (0, _forEach3.default)(aPoints.h, function (x1, i) {
                (0, _forEach3.default)(bPoints.h, function (x2) {
                    if (Math.abs(x1 - x2) <= offset / self.zoom) {
                        ret.left = x2;
                        ret._H = x1 - x2;
                        ret._edgeH = i;
                        return false;
                    }
                });

                if (ret.left !== HIDE_OFFSET) {
                    return false;
                }
            });

            (0, _forEach3.default)(aPoints.v, function (x1, i) {
                (0, _forEach3.default)(bPoints.v, function (x2) {
                    if (Math.abs(x1 - x2) <= offset / self.zoom) {
                        ret.top = x2;
                        ret._V = x1 - x2;
                        ret._edgeV = i;
                        return false;
                    }
                });

                if (ret.top !== HIDE_OFFSET) {
                    return false;
                }
            });

            return ret;
        },
        getBBoxList: function getBBoxList() {
            var model = this.model;
            var layout = this.layout;
            var elements = layout.elements;
            if (!model || !elements || !elements.length) {
                return;
            }

            var list = [{
                rotate: 0,
                height: layout.height,
                width: layout.width,
                left: 0,
                top: 0
            }];

            return list.concat(elements).filter(function (element) {
                return element !== model && !element.$selected;
            }).map(function (element) {
                return _utils2.default.getBBox(element);
            });
        },
        checkGuidePoints: function checkGuidePoints(e) {
            var self = this;
            var bboxList = this.bboxList;
            if (!bboxList || !bboxList.length) {
                return;
            }
            var modelBBox = _utils2.default.getBBox(this.model);

            self.top = HIDE_OFFSET;
            self.left = HIDE_OFFSET;

            var model = self.model;

            var gapValueV = 0;
            var gapValueH = 0;

            model.$snapValue = {
                width: 0,
                height: 0,
                left: 0,
                top: 0
            };

            (0, _forEach3.default)(bboxList, function (bbox) {
                var point = self.getGuidePoint(modelBBox, bbox);

                if (self.left === HIDE_OFFSET && point.left !== HIDE_OFFSET) {
                    self.left = point.left;
                    if (!e.ctrlKey && model.$guider.snapTo) {
                        var actionData = self.actionData;
                        var dir = actionData && actionData.dir;

                        if (dir) {
                            if (point._edgeH === 1) {
                                point._H *= 2;
                            }
                            gapValueV = point._H / (model.width / model.height);

                            if (~dir.indexOf('e')) {
                                model.width -= point._H;

                                if (~dir.indexOf('n')) {
                                    model.height -= gapValueV;
                                    model.top += gapValueV;
                                } else if (~dir.indexOf('s')) {
                                    model.height -= gapValueV;
                                }
                            } else if (~dir.indexOf('w')) {
                                model.width += point._H;
                                model.left -= point._H;

                                if (~dir.indexOf('n')) {
                                    model.height += gapValueV;
                                    model.top -= gapValueV;
                                } else if (~dir.indexOf('s')) {
                                    model.height += gapValueV;
                                }
                            }
                        } else {
                            model.left -= point._H;
                            model.$snapValue.left = -point._H;
                        }
                    }
                }
                if (self.top === HIDE_OFFSET && point.top !== HIDE_OFFSET) {
                    self.top = point.top;

                    if (!e.ctrlKey && model.$guider.snapTo) {
                        var _actionData = self.actionData;
                        var _dir = _actionData && _actionData.dir;

                        if (_dir) {
                            if (point._edgeV === 1) {
                                point._V *= 2;
                            }
                            gapValueH = point._V * (model.width / model.height);
                            if (gapValueV !== 0 && gapValueH !== 0 || point._edgeH === 1) {
                                return;
                            }

                            if (~_dir.indexOf('s')) {
                                model.height -= point._V;

                                if (~_dir.indexOf('e')) {
                                    model.width -= gapValueH;
                                } else if (~_dir.indexOf('w')) {
                                    model.width -= gapValueH;
                                    model.left += gapValueH;
                                }
                            } else if (~_dir.indexOf('n')) {
                                model.height += point._V;
                                model.top -= point._V;

                                if (~_dir.indexOf('e')) {
                                    model.width += gapValueH;
                                } else if (~_dir.indexOf('w')) {
                                    model.width += gapValueH;
                                    model.left -= gapValueH;
                                }
                            }
                        } else {
                            model.top -= point._V;
                            model.$snapValue.top = -point._V;
                        }
                    }
                }

                if (self.left !== HIDE_OFFSET && self.top !== HIDE_OFFSET) {
                    return false;
                }

                return true;
            });
        }
    },
    events: {
        'editor.drag.active': function editorDragActive(model, data) {
            var _this = this;

            if (model.$guider.show) {
                this.model = model;
                this.actionData = data;

                model.$snapValue = {
                    width: 0,
                    height: 0,
                    left: 0,
                    top: 0
                };

                setTimeout(function () {
                    if (model.snapBboxList) {
                        _this.bboxList = [].concat(model.snapBboxList);
                    } else {
                        _this.bboxList = _this.getBBoxList();
                    }
                }, 0);
            }
        },
        'editor.drag.moving': function editorDragMoving(drag, model, e) {
            if (model.$guider.show) {
                this.checkGuidePoints(e);
            }
        },
        'editor.drag.inactive': function editorDragInactive() {
            this.left = HIDE_OFFSET;
            this.top = HIDE_OFFSET;
            this.model = null;
            this.actionData = null;
            this.bboxList = null;
        }
    },

    beforeDestroy: function beforeDestroy() {
        this.$emit('destroy');
    }
};
module.exports = exports['default'];