'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _editorHighlightElement = require('./editor-highlight-element.html');

var _editorHighlightElement2 = _interopRequireDefault(_editorHighlightElement);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

;

exports.default = {
    template: _editorHighlightElement2.default,
    props: ['currentLayout'],
    data: function data() {
        return {
            elements: []
        };
    },


    computed: {
        editor: function editor() {
            return this.$parent;
        },
        highlights: function highlights() {
            var _this = this;

            var editor = this.editor;
            var containerRect = editor.containerRect;
            var shellRect = editor.shellRect;
            var zoom = editor.zoom;

            if (!this.currentLayout) {
                return [];
            }

            return this.elements.map(function (element) {
                return {
                    left: shellRect.left + containerRect.scrollLeft + element.left * zoom,
                    top: shellRect.top + containerRect.scrollTop + (element.top + _this.currentLayout.top) * zoom,
                    width: element.width * zoom,
                    height: element.height * zoom,
                    transform: element.transform.toString(),
                    className: element.$highlightClass
                };
            });
        }
    },

    methods: {
        add: function add(appends, className) {
            var elements = this.elements;
            appends = [].concat(appends);
            appends.forEach(function (element) {
                if (elements.indexOf(element) === -1) {
                    elements.push(element);
                }
                element.$highlightClass = className;
            });
        },
        replaceAll: function replaceAll(newElems, className) {
            this.clear();
            this.add(newElems, className);
        },
        remove: function remove(removes) {
            var elements = this.elements;
            removes = [].concat(removes);
            removes.forEach(function (element) {
                var index = elements.indexOf(element);
                if (index > -1) {
                    elements.splice(index, 1);
                    delete element.$highlightClass;
                }
            });
        },
        clear: function clear() {
            this.elements.forEach(function (element) {
                delete element.$highlightClass;
            });
            this.elements = [];
        }
    },

    events: {
        'editor.highlight.element': function editorHighlightElement(elements) {
            var action = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'replaceAll';
            var className = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '';

            if (elements) {
                this[action](elements, className);
            } else {
                this.clear();
            }
        }
    }
};
module.exports = exports['default'];