'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _merge2 = require('lodash/merge');

var _merge3 = _interopRequireDefault(_merge2);

var _noop2 = require('lodash/noop');

var _noop3 = _interopRequireDefault(_noop2);

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _jquery = require('jquery');

var _jquery2 = _interopRequireDefault(_jquery);

var _vueInherit = require('../utils/vue-inherit');

var _vueInherit2 = _interopRequireDefault(_vueInherit);

var _editorTextBase = require('../canvas/editor-text-base');

var _editorTextBase2 = _interopRequireDefault(_editorTextBase);

var _editorTextEditor = require('./editor-text-editor.html');

var _editorTextEditor2 = _interopRequireDefault(_editorTextEditor);

var _richText = require('../utils/rich-text');

var _richText2 = _interopRequireDefault(_richText);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var doc = (0, _jquery2.default)(document);
var rVoidWidth = /\u200b/gm;

exports.default = (0, _vueInherit2.default)(_editorTextBase2.default, {
    template: _editorTextEditor2.default,
    props: ['global', 'model', 'options'],
    data: function data() {
        return {
            clickLocked: false,

            html: ''
        };
    },

    methods: {
        syncRect: _noop3.default,

        initTextEditor: function initTextEditor() {
            var _this2 = this;

            this.createRichText();
            this.html = this.contentsHTML;

            this.$on('destroy', function () {
                _this2._isChange && _this2.$events.$emit('editor.edit.apply', _this2.model);
                _this2.model.$editing = false;
                _this2.$events.$emit('editor.currentRichText.update', null);

                _this2._richText.clear();
            });
        },
        createRichText: function createRichText() {
            var _this3 = this;

            this._richText = new _richText2.default(this.$refs.edit, this.richTextOptions);

            this.$events.$emit('editor.currentRichText.update', this._richText);

            this._richText.cmd.on('do', function (e, name, value) {
                if (!_this3.model.$editing) return;
                if (name === 'fontFamily') {
                    _this3.load(value);
                }
            });

            this._richText.on('changeImmediate', function (e, textElem) {
                var html = textElem.innerHTML;

                if (html !== _this3._prevHtml) {
                    _this3.setContents(_this3.model);
                }
                _this3._prevHtml = html;
            });
        },
        setContents: function setContents(model) {
            if (!this._richText || !model || model.type !== 'text' || !model.$editing) return;

            var contents = this._richText.fromJSON(null, {}, this.global.zoom);

            var mergeStyleProps = _richText2.default.getAllEqualStyleProps(contents);

            model.contents = contents;

            model.content = this.$refs.edit.textContent;

            (0, _merge3.default)(model, mergeStyleProps);

            this._isChange = true;
        },
        checkFocus: function checkFocus() {
            var _this4 = this;

            var model = this.model;

            if (!model.$editing || !this._richText) return;
            this.$nextTick(function () {
                var $textElem = (0, _jquery2.default)(_this4.$refs.edit);
                var text = $textElem.text().replace(rVoidWidth, '');

                if (!text) {
                    _this4._richText.selection.createRangeByElem($textElem.children().last()[0] || $textElem[0], true);
                    _this4._richText.selection.collapse();
                } else {
                    _this4._richText.selection.createRangeByElem($textElem[0], true);
                }
            });
        },
        onMousedown: function onMousedown() {
            var _this = this;
            var delayFn = function delayFn() {
                setTimeout(function () {
                    _this.clickLocked = false;
                }, 160);
            };

            this.clickLocked = true;

            doc.one('mouseup.editor-text-editor', delayFn);
        },

        checkLoad: _noop3.default
    },
    events: {
        'editor.click': function editorClick(e) {
            if (this.clickLocked) {
                e.preventDefault();
            }
        },
        'editor.keydown': function editorKeydown(e) {
            var model = this.model;

            if (model && e.keyCode === 27) {
                model.$editing = false;
            }
        },
        'editor.edit.apply': function editorEditApply(model) {
            model = model || this.model;
            this.setContents(model);
            model.$editing = false;
        },
        'editor.zoom': function editorZoom() {
            this.html = this.contentsHTML;
            this.checkFocus();
        },
        'editor.transform.resizing': function editorTransformResizing() {
            this.html = this.contentsHTML;
        }
    },
    computed: {
        richTextOptions: function richTextOptions() {
            return this.model && {
                defaultStyle: _extends({}, this.model, {
                    fontSize: this.model.fontSize * this.global.zoom,
                    fontFamily: this.fontFamily
                })
            };
        }
    },
    watch: {
        'model.$editing': function model$editing(v) {
            if (v) this.html = this.contentsHTML;

            this.checkFocus();
        },
        'model': function model(newModel, oldModel) {
            if (oldModel) {
                this._isChange && this.$events.$emit('editor.edit.apply', oldModel);
                oldModel.$editing = false;
            }

            this._isChange = false;

            this.html = this.contentsHTML;
        },
        'richTextOptions': function richTextOptions() {
            if (this._richText && this.model) {
                this._richText.setConfig(this.richTextOptions);
            }
        }
    },

    mounted: function mounted() {
        this.initTextEditor();

        this.checkFocus();
    }
});
module.exports = exports['default'];