'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _noop2 = require('lodash/noop');

var _noop3 = _interopRequireDefault(_noop2);

var _assign2 = require('lodash/assign');

var _assign3 = _interopRequireDefault(_assign2);

var _merge2 = require('lodash/merge');

var _merge3 = _interopRequireDefault(_merge2);

var _jquery = require('jquery');

var _jquery2 = _interopRequireDefault(_jquery);

var _utils = require('../utils/utils');

var _utils2 = _interopRequireDefault(_utils);

var _vueInherit = require('../utils/vue-inherit');

var _vueInherit2 = _interopRequireDefault(_vueInherit);

var _matrix = require('../utils/matrix');

var _matrix2 = _interopRequireDefault(_matrix);

var _transform = require('../utils/transform');

var _transform2 = _interopRequireDefault(_transform);

var _editorElement = require('../canvas/editor-element');

var _editorElement2 = _interopRequireDefault(_editorElement);

var _editorTransform = require('./editor-transform.html');

var _editorTransform2 = _interopRequireDefault(_editorTransform);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var doc = (0, _jquery2.default)(document);

exports.default = (0, _vueInherit2.default)(_editorElement2.default, {
    props: ['options', 'global', 'model', 'currentLayout', 'subModel'],
    template: _editorTransform2.default,
    data: function data() {
        return {
            gripDirs: ['s', 'sw', 'w', 'nw', 'n', 'ne', 'e', 'se'],
            clickLocked: false,
            dragLocked: false,
            rotateReady: false,

            innerStyleVal: {}
        };
    },

    computed: {
        editor: function editor() {
            return this.$parent;
        },
        innerStyle: function innerStyle() {
            var global = this.global;
            var clip = this.editor.currentElement.clipModel;
            if (!clip) {
                clip = { top: 0, left: 0, right: 0, bottom: 0 };
            }

            if (this.model.type === '$croper') {
                return {
                    position: 'absolute',
                    zIndex: 2,
                    left: global.zoom * clip.left - 1 + 'px',
                    top: global.zoom * clip.top - 1 + 'px',
                    right: global.zoom * clip.right + 'px',
                    bottom: global.zoom * clip.bottom + 'px'
                };
            } else {
                return this.innerStyleVal;
            }
        },
        transform: function transform() {
            return this.model.transform.toString();
        },
        transformInvert: function transformInvert() {
            var transform = this.model.transform;
            var _transform$localTrans = transform.localTransform,
                a = _transform$localTrans.a,
                b = _transform$localTrans.b,
                c = _transform$localTrans.c,
                d = _transform$localTrans.d,
                tx = _transform$localTrans.tx,
                ty = _transform$localTrans.ty;

            var matrix = new _matrix2.default(a, b, c, d, tx, ty);
            matrix.invert();
            var invertTransform = new _transform2.default();
            invertTransform.setFromMatrix(matrix);
            return invertTransform.toString();
        },
        skewInvert: function skewInvert() {
            var _model = this.model,
                skewX = _model.skewX,
                skewY = _model.skewY;

            var invertX = -_utils2.default.radToDeg(skewX);
            var invertY = -_utils2.default.radToDeg(skewY);
            return 'skew(' + invertX + 'deg, ' + invertY + 'deg)';
        },
        rotateInvert: function rotateInvert() {
            return 'rotate(' + (360 - this.model.rotate) + 'deg)';
        },
        octant: function octant() {
            var rotate = this.model.rotate;
            return 'octant' + Math.floor((rotate + 22.5) / 45);
        },
        cropperInner: function cropperInner() {
            return this.model.$name === '$cropperInner';
        }
    },
    methods: {
        initResize: function initResize(e, dir) {
            var self = this;
            var model = this.model;
            var zoom = self.editor.zoom;

            if (e.button !== 0 || model.$draging) {
                return;
            }


            var drag = this.drag = {
                dir: dir,
                rotate: model.rotate,
                height: model.height,
                width: model.width,
                left: model.left,
                top: model.top,
                pageX: e.pageX,
                pageY: e.pageY,
                clip: model.clip ? (0, _merge3.default)({}, this.rect.clip) : null,
                points: _utils2.default.getPointsByElement(model),

                minHeight: 0,
                minWidth: 0,
                minLeft: 0,
                minTop: 0,
                maxLeft: 0,
                maxTop: 0,
                move: function move(e) {
                    e.preventDefault();

                    var dx = e.pageX - drag.pageX;
                    var dy = e.pageY - drag.pageY;
                    var dir = drag.dir;

                    var dxy = _utils2.default.getDxyByAngle(dx, dy, drag.rotate);

                    dxy.dx = Math.round(dxy.dx);
                    dxy.dy = Math.round(dxy.dy);

                    if (!/[ew]/.test(dir)) {
                        dxy.dx = 0;
                    }
                    if (!/[ns]/.test(dir)) {
                        dxy.dy = 0;
                    }

                    if (dir.length > 1 && (!e.shiftKey || model.type === 'text' && model.autoScale)) {
                        var ratio = drag.height / drag.width || 1;
                        if (dir === 'ne' || dir === 'sw') {
                            ratio *= -1;
                        }

                        dxy.dy = dxy.dx * ratio;
                    }

                    drag.dx = dxy.dx / zoom;
                    drag.dy = dxy.dy / zoom;

                    if (!drag.draging) {
                        drag.draging = true;

                        self.$events.$emit('editor.transform.active', model, {
                            action: 'resize', dir: dir
                        });
                    }

                    self.$events.$emit('editor.transform.resizing', drag, model);
                },
                cancel: function cancel() {
                    doc.off('mousemove', drag.move);

                    self.drag = null;
                    model.$draging = false;

                    if (drag.draging) {
                        self.$events.$emit('editor.transform.inactive', model, drag, {
                            action: 'resize'
                        });

                        drag.draging = false;
                    }
                }
            };

            model.$draging = true;

            doc.on('mousemove', drag.move);
            doc.one('mouseup', drag.cancel);
        },
        initRotaor: function initRotaor(e) {
            var self = this;
            var model = this.model;

            if (e.button !== 0 || model.$draging) {
                return;
            }

            this.rotateReady = true;

            var drag = this.drag = {
                dotX: 0,
                dotY: 0,
                pageX: 0,
                pageY: 0,
                rotate: 0,
                move: function move(e) {
                    e.preventDefault();

                    var rotate = _utils2.default.getAngle(e.pageX, e.pageY, drag.dotX, drag.dotY, 90);

                    if (e.shiftKey) {
                        rotate = Math.round(rotate / 10) * 10;
                    }

                    if (!e.ctrlKey) {
                        [{ angle: 30, edge: 2 }, { angle: 45, edge: 3 }].some(function (item) {
                            var angle = item.angle;
                            var edge = item.edge;
                            var closest = Math.round(rotate / angle) * angle;

                            if (Math.abs(closest - rotate) < edge) {
                                rotate = closest;
                                return true;
                            }
                        });
                    }

                    drag.rotate = rotate;

                    if (!drag.draging) {
                        drag.draging = true;

                        self.$events.$emit('editor.transform.active', model, {
                            action: 'rotate'
                        });
                    }

                    self.$events.$emit('editor.transform.rotating', drag, model);
                },
                cancel: function cancel() {
                    doc.off('mousemove', drag.move);

                    self.drag = null;
                    model.$draging = false;

                    if (drag.draging) {
                        self.$events.$emit('editor.transform.inactive', model, drag, {
                            action: 'rotate'
                        });

                        drag.draging = false;
                    }

                    self.rotateReady = false;
                }
            };

            var zoom = this.editor.zoom;
            var element = (0, _jquery2.default)(e.currentTarget || e.target);
            var shell = element.closest('.editor-shell');
            var shellOffset = shell.offset();

            var height = zoom * model.height;
            var width = zoom * model.width;
            var left = zoom * model.left;
            var top = zoom * (model.top + self.currentLayout.top);

            (0, _assign3.default)(drag, {
                dotX: shellOffset.left + left + width / 2,
                dotY: shellOffset.top + top + height / 2,
                pageX: e.pageX,
                pageY: e.pageY
            });

            model.$draging = true;

            doc.on('mousemove', drag.move);
            doc.one('mouseup', drag.cancel);
        },
        checkClickAlive: function checkClickAlive(e) {
            var rSelfClass = /(?:^|\s)editor-transform(?:$|\s)/;
            var model = this.model;

            var alive = true;

            if (model.lock || this.dragLocked || model.type.indexOf('$') === 0 || !rSelfClass.test(e.target.className)) {
                alive = false;
            }

            return alive;
        },
        lockEditorClick: function lockEditorClick() {
            var _this = this;

            this.clickLocked = true;

            clearTimeout(this.lockEditorClickTimer);
            this.lockEditorClickTimer = setTimeout(function () {
                _this.clickLocked = false;
            }, 160);
        },
        checkClickAdp: function checkClickAdp(e) {
            if (!this.checkClickAlive(e)) {
                return;
            }

            var self = this;
            var dblclickDelay = 256;

            this.lockEditorClick();

            this.clickCount = (this.clickCount || 0) + 1;

            if (this.clickCount === 2) {
                this.$events.$emit('transform.dblclick', e);
            }

            clearTimeout(this.checkClickAdpTimer);
            this.checkClickAdpTimer = setTimeout(function () {
                var clickCount = self.clickCount;

                self.clickCount = 0;

                if (clickCount === 1) {
                    self.$events.$emit('transform.click', e);
                }
            }, dblclickDelay);
        },
        focusElementByEvent: function focusElementByEvent(e) {
            var editor = this.editor;
            var point = editor.pointFromEvent(e);

            return editor.focusElementByPoint(point.x, point.y);
        },
        innerClick: function innerClick(e) {
            if (this.options.cropperOptions.defaultTarget === 'outer') {
                return;
            }

            if (e.button === 0) {
                if (this.innerActive && e.type === 'mouseup') {
                    var dx = Math.abs(this.innerActive.pageX - e.pageX);
                    var dy = Math.abs(this.innerActive.pageY - e.pageY);
                    if (dx < 3 && dy < 3) {
                        this.$events.$emit('imageCroper.activeInner');
                    }
                    this.innerActive = false;
                } else if (!this.innerActive && e.type === 'mousedown') {
                    this.innerActive = e;
                } else {
                    this.innerActive = false;
                }
            }
        },
        getBoundingClientRect: function getBoundingClientRect() {
            var $els = this.$refs;
            var transformEleRect = $els.transformEle.getBoundingClientRect();
            var rotatorHandleRect = $els.rotatorHandle.getBoundingClientRect();

            if (rotatorHandleRect.width === 0 || rotatorHandleRect.height === 0) {
                rotatorHandleRect = transformEleRect;
            }

            var rect = {
                top: Math.min(transformEleRect.top, rotatorHandleRect.top),
                left: Math.min(transformEleRect.left, rotatorHandleRect.left),
                bottom: Math.max(transformEleRect.bottom, rotatorHandleRect.bottom),
                right: Math.max(transformEleRect.right, rotatorHandleRect.right),

                _transformEleRect: transformEleRect,
                _rotatorHandleRect: rotatorHandleRect
            };

            rect.width = rect.right - rect.left;
            rect.height = rect.bottom - rect.top;

            return rect;
        },

        checkLoad: _noop3.default
    },
    events: {
        'editor.click': function editorClick(e) {
            if (this.clickLocked || this.dragLocked) {
                if (!e.shiftKey && !e.metaKey) {
                    e.preventDefault();
                }

                this.clickLocked = false;
                this.dragLocked = false;
            }

            return true;
        },
        'transform.click': function transformClick(e) {
            if (!e.shiftKey && !e.metaKey) {
                this.focusElementByEvent(e);
            }
        },
        'transform.dblclick': function transformDblclick(e) {
            var editor = this.editor;

            this.focusElementByEvent(e);

            var element = editor.currentSubElement || editor.currentElement;

            switch (element.type) {
                case 'image':
                case 'mask':
                    if (element.editable) {
                        if (this.options.hookImagePicker) {
                            this.$events.$emit('editor.element.picker');
                        } else {
                            editor.pickImage(function (image, url) {
                                editor.replaceImage(url, {
                                    width: image.width,
                                    height: image.height
                                }, element);
                            }, function () {
                                element.$loaded = false;
                            });
                        }
                    }
                    break;
                case 'text':
                    editor.showElementEditor(element);
                    break;

                case 'svg':
                    this.$events.$emit('editor.element.dblclick', element);
                    break;
            }

            e.stopPropagation();
        },
        'editor.transform.active': function editorTransformActive() {
            this.dragLocked = true;
        },
        'editor.transform.inactive': function editorTransformInactive() {
            var _this2 = this;

            setTimeout(function () {
                _this2.dragLocked = false;
            }, 160);
        },
        'transform.drag': function transformDrag(e) {
            (0, _jquery2.default)(this.$el).trigger('mousedown.editor-dragger', e);
        },
        'editor.croper.update': function editorCroperUpdate(val) {
            this.innerStyleVal = val;
        }
    },

    filters: {
        angle: function angle(_angle) {
            return Math.round(_angle) % 360 + '°';
        }
    }
});
module.exports = exports['default'];