'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _throttle2 = require('lodash/throttle');

var _throttle3 = _interopRequireDefault(_throttle2);

var _merge2 = require('lodash/merge');

var _merge3 = _interopRequireDefault(_merge2);

var _assign2 = require('lodash/assign');

var _assign3 = _interopRequireDefault(_assign2);

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _jquery = require('jquery');

var _jquery2 = _interopRequireDefault(_jquery);

var _tinycolor = require('tinycolor2');

var _tinycolor2 = _interopRequireDefault(_tinycolor);

var _editorCore = require('./editor-core');

var _editorCore2 = _interopRequireDefault(_editorCore);

var _editorDefaults = require('./editor-defaults');

var _editorDefaults2 = _interopRequireDefault(_editorDefaults);

var _config = require('./utils/rich-text/config');

var _utils = require('./utils/utils');

var _utils2 = _interopRequireDefault(_utils);

var _hotkeys = require('@gaoding/hotkeys');

var _pasteImage = require('./utils/paste-image');

var _pasteImage2 = _interopRequireDefault(_pasteImage);

var _dropFile = require('./utils/drop-file');

var _dropFile2 = _interopRequireDefault(_dropFile);

var _vueInherit = require('./utils/vue-inherit');

var _vueInherit2 = _interopRequireDefault(_vueInherit);

var _pickImage2 = require('./utils/pick-image');

var _pickImage3 = _interopRequireDefault(_pickImage2);

var _loader = require('./utils/loader');

var _loader2 = _interopRequireDefault(_loader);

var _editor = require('./editor.html');

var _editor2 = _interopRequireDefault(_editor);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var view = (0, _jquery2.default)(window);

exports.default = (0, _vueInherit2.default)(_editorCore2.default, {
    template: _editor2.default,
    data: function data() {
        return {
            containerRect: {
                topOffset: 0,
                scrollLeft: 0,
                scrollTop: 0,
                scrollWidth: 0,
                scrollHeight: 0,
                padding: 0,
                height: 0,
                width: 0,
                left: 0,
                top: 0
            },
            shellRect: {
                height: 0,
                width: 0,
                left: 0,
                top: 0
            },

            currentCropElement: null,

            currentEditMask: null,

            currentRichText: null,

            currentTextSelection: null
        };
    },

    computed: {
        'zoomWidthHeight': function zoomWidthHeight() {
            return this.zoom + this.height + this.width;
        },
        'currentSelection': function currentSelection() {
            return this.currentTextSelection;
        }
    },
    methods: {
        zoomTo: function zoomTo(zoom, point) {
            var _this = this;

            var _preZoom = this.zoom;
            this.zoom = zoom;

            var changeOffset = {
                width: this.width * (this.zoom - _preZoom),
                height: this.height * (this.zoom - _preZoom)
            };

            var containerRect = this.containerRect;
            var shellRect = this.shellRect;

            if (!point) {
                point = {
                    x: containerRect.width / 2,
                    y: containerRect.height / 2
                };
            }
            point._perX = point.x / containerRect.width;
            point._perY = point.y / containerRect.height;

            var leaveRight = changeOffset.width * (1 - point._perX) + (shellRect.width + containerRect.padding * 2 - (containerRect.width + containerRect.scrollLeft));

            var leaveBottom = changeOffset.height * (1 - point._perY) + (shellRect.height + containerRect.padding * 2 - (containerRect.height + containerRect.scrollTop));

            var scrollLeft = containerRect.scrollLeft;
            var scrollTop = containerRect.scrollTop;

            this.$events.$emit('editor.zoom.before');

            this.$nextTick(function () {
                _this.container.scrollLeft(scrollLeft + changeOffset.width * point._perX - (leaveRight < 0 ? leaveRight : 0)).scrollTop(scrollTop + changeOffset.height * point._perY - (leaveBottom < 0 ? leaveBottom : 0));

                _this.$events.$emit('editor.zoom', zoom);
            });
        },
        fitZoom: function fitZoom() {
            var layout = this.currentLayout;
            if (!layout || this.mode === 'mirror') {
                return;
            }

            var layoutWidth = layout.width;
            var layoutHeight = layout.height;
            var containerRect = this.containerRect;
            var padding = containerRect.padding;
            var rectWidth = containerRect.width - 2 * padding;
            var rectHeight = containerRect.height - 2 * padding;

            if (layoutWidth <= 0 || layoutHeight <= 0 || rectWidth <= 0 || rectHeight <= 0) {
                return;
            }

            var heightRatio = layoutHeight / rectHeight;
            var widthRatio = layoutWidth / rectWidth;
            var ratio = widthRatio;

            if (heightRatio > ratio) {
                ratio = heightRatio;
            }

            this.zoomTo(Math.min(1, 1 / ratio));
        },
        addImage: function addImage(url, options) {
            var data = (0, _assign3.default)({
                type: 'image'
            }, options);

            data.url = url || this.options.placeImage;

            return this.addElement(data);
        },
        addImageEffect: function addImageEffect() {
            var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
            var element = arguments[1];

            element = this.getElement(element);
            if (!element || element.type !== 'image') {
                return;
            }

            var effect = (0, _merge3.default)({}, _editorDefaults2.default.imageEffect, options);

            element.imageEffects.push(effect);
            this.makeSnapshot('change_image_effect');
        },
        removeImageEffect: function removeImageEffect(effect, element) {
            element = this.getElement(element);
            if (!effect || !element || element.type !== 'image') {
                return;
            }

            var index = element.imageEffects.indexOf(effect);
            if (index === -1) {
                return;
            }

            element.imageEffects.splice(index, 1);
            this.makeSnapshot('change_image_effect');
        },
        changeImageEffect: function changeImageEffect(effect) {
            var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
            var element = arguments[2];

            element = this.getElement(element);
            if (!effect || !element || element.type !== 'image') {
                return;
            }

            var index = element.imageEffects.findIndex(function (target) {
                return target === effect;
            });
            if (index === -1) {
                return;
            }

            var newEffect = (0, _merge3.default)({}, _editorDefaults2.default.imageEffect, { filling: { color: element.color } }, options);
            element.imageEffects.splice(index, 1, newEffect);
            this.makeSnapshot('change_image_effect');
        },
        addText: function addText(text, options) {
            var data = (0, _assign3.default)({
                type: 'text',
                fontSize: 16,
                fontFamily: 'Simsun',
                content: text
            }, options);

            var widthOffset = 10;
            var heightOffset = 10;

            if (!data.width) {
                var panel = this.$el;
                var span = document.createElement('span');
                var cssText = 'position:absolute;left:0;top:-100px;';
                var zoom = this.global.zoom;


                cssText += 'white-space: nowrap;';
                cssText += 'max-width:' + this.width / zoom + 'px;';
                cssText += 'line-height:' + (data.lineHeight || 'normal') + ';';
                cssText += 'font-size:' + data.fontSize / zoom + 'px;';

                if (data.fontFamily) {
                    cssText += 'font-family:' + data.fontFamily;
                }

                span.innerHTML = text;
                span.style.cssText = cssText;
                panel.appendChild(span);

                data.width = span.offsetWidth * zoom + widthOffset;
                data.height = span.offsetHeight * zoom + heightOffset;

                if (!data.lineHeight) {
                    data.lineHeight = data.height / data.fontSize;
                }

                panel.removeChild(span);
            }

            return this.addElement(data);
        },
        addTextEffect: function addTextEffect() {
            var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
            var element = arguments[1];

            element = this.getElement(element);
            if (!element || element.type !== 'text') {
                return;
            }

            var effect = (0, _merge3.default)({}, _editorDefaults2.default.textEffect, { filling: { color: element.color } }, options);
            element.textEffects.push(effect);
            this.makeSnapshot('change_text_effect');
        },
        removeTextEffect: function removeTextEffect(effect, element) {
            element = this.getElement(element);
            if (!effect || !element || element.type !== 'text') {
                return;
            }

            var index = element.textEffects.indexOf(effect);
            if (index === -1) {
                return;
            }

            element.textEffects.splice(index, 1);
            this.makeSnapshot('change_text_effect');
        },
        removeAllTextEffects: function removeAllTextEffects(element) {
            element = this.getElement(element);
            if (!element || element.type !== 'text' || !element.textEffects.length) {
                return;
            }

            element.textEffects = [];
            this.makeSnapshot('change_text_effect');
        },
        changeTextEffect: function changeTextEffect(effect) {
            var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
            var element = arguments[2];

            element = this.getElement(element);
            if (!effect || !element || element.type !== 'text') {
                return;
            }

            var index = element.textEffects.findIndex(function (target) {
                return target === effect;
            });
            if (index === -1) {
                return;
            }

            var newEffect = (0, _merge3.default)({}, _editorDefaults2.default.textEffect, { filling: { color: element.color } }, options);
            element.textEffects.splice(index, 1, newEffect);
            this.makeSnapshot('change_text_effect');
        },
        changeTextContents: function changeTextContents(props, element) {
            var _this2 = this;

            element = this.getElement(element);

            var richText = this.currentRichText;

            if (element.type !== 'text') return;

            if (!element.$editing) {
                element.contents = _utils.serialize.injectStyleProps(element.contents, props);
                (0, _merge3.default)(element, props);
                this.makeSnapshot('change_text_contents');
            } else if (richText) {
                Object.keys(props).forEach(function (name) {
                    return richText.cmd.do(name, name === 'fontSize' ? props[name] * _this2.zoom : props[name]);
                });
            }
        },
        changeEffectColor: function changeEffectColor(newColor, index, element) {
            var targetElement = element || this.currentElement;
            if (!targetElement || targetElement.type !== 'text' && targetElement.type !== 'image') {
                return;
            }

            var aggregatedColors = targetElement.aggregatedColors;

            if (targetElement.mainColor !== null && index === undefined) {
                var _effects = targetElement.type === 'text' ? targetElement.textEffects : targetElement.imageEffects;

                var mainHsl = (0, _tinycolor2.default)(targetElement.mainColor).toHsl();
                var toHsl = (0, _tinycolor2.default)(newColor).toHsl();

                if (!targetElement.$hslDiff) {
                    targetElement.$hslDiff = targetElement.aggregatedColors.map(function (color) {
                        var baseHsl = (0, _tinycolor2.default)(color).toHsl();
                        return {
                            h: baseHsl.h - mainHsl.h,
                            s: baseHsl.s - mainHsl.s,
                            l: baseHsl.l - mainHsl.l
                        };
                    });
                }

                var clamp = function clamp(x) {
                    return Math.max(0, Math.min(1, x));
                };

                var circular = function circular(x) {
                    return (360 + x % 360) % 360;
                };
                var diffFrom = function diffFrom(linkIndex) {
                    if (linkIndex === undefined) {
                        return;
                    }

                    var $hslDiff = targetElement.$hslDiff;

                    var newHsl = {
                        h: circular(toHsl.h + $hslDiff[linkIndex].h),
                        s: clamp(toHsl.s + $hslDiff[linkIndex].s),
                        l: clamp(toHsl.l + $hslDiff[linkIndex].l)
                    };
                    return (0, _tinycolor2.default)(newHsl).toHex8String();
                };

                _effects.forEach(function (effect) {
                    ['stroke', 'shadow', 'filling'].forEach(function (type) {
                        if (effect[type].$linkIndex !== undefined) {
                            effect[type].color = diffFrom(effect[type].$linkIndex);
                        }
                    });
                });

                targetElement.mainColor = (0, _tinycolor2.default)(toHsl).toHex8String();

                if (targetElement.$linkIndex !== undefined) {
                    var _newColor = diffFrom(targetElement.$linkIndex);
                    targetElement.color = _newColor;
                    this.changeTextContents({ color: _newColor }, targetElement);
                }
                targetElement.aggregatedColors = aggregatedColors.map(function (color, i) {
                    return diffFrom(i);
                });

                this.makeSnapshot('change_effect_color');
                return;
            }

            var effects = targetElement.type === 'text' ? targetElement.textEffects : targetElement.imageEffects;
            effects.forEach(function (effect) {
                ['stroke', 'shadow', 'filling'].forEach(function (type) {
                    if (effect[type].$linkIndex === index) {
                        effect[type].color = newColor;
                    }
                });
            });

            if (targetElement.$linkIndex === index) {
                targetElement.color = newColor;
                this.changeTextContents({ color: newColor });
            }

            this.$set(targetElement.aggregatedColors, index, newColor);

            this.makeSnapshot('change_effect_color');
        },
        changeEffectScale: function changeEffectScale(newScale, element) {
            var targetElement = element || this.currentElement;
            if (!targetElement || targetElement.type !== 'text' && targetElement.type !== 'image') {
                return;
            }

            var ratio = newScale / targetElement.effectScale;
            var effects = targetElement.type === 'text' ? targetElement.textEffects : targetElement.imageEffects;
            effects.forEach(function (effect) {
                var offset = effect.offset,
                    stroke = effect.stroke,
                    shadow = effect.shadow;

                if (!effect.excludeScale.includes('offset')) {
                    offset.x *= ratio;
                    offset.y *= ratio;
                }
                if (!effect.excludeScale.includes('stroke')) {
                    stroke.width *= ratio;
                }
                if (!effect.excludeScale.includes('shadow')) {
                    shadow.offsetX *= ratio;
                    shadow.offsetY *= ratio;
                }
            });

            targetElement.effectScale = newScale;
            this.makeSnapshot('change_effect_scale');
        },
        flipElement: function flipElement(dir, element) {
            element = this.getElement(element);
            if (!element) {
                return;
            }

            var props = {};
            var name = dir === 'x' ? 'scaleX' : 'scaleY';

            props[name] = -1 * element[name];

            this.changeElement(props, element);
        },
        getOverlapElements: function getOverlapElements(element) {
            if (!element) {
                element = this.currentElement;
            }

            var resultElements = [];
            this.currentLayout.elements.forEach(function (layoutElement) {
                if (_utils2.default.getRectIntersection(element, layoutElement)) {
                    resultElements.push(layoutElement);
                }
            });
            return resultElements;
        },
        showElementBlink: function showElementBlink(element) {
            element = this.getElement(element);
            if (!element) {
                return;
            }
            element.$blinking = true;
        },
        hideElementBlink: function hideElementBlink(element) {
            element = this.getElement(element);
            if (!element) {
                return;
            }
            element.$blinking = false;
        },
        showElementEditor: function showElementEditor(element) {
            element = this.getElement(element);
            if (!element) {
                return;
            }

            var type = element.type;
            if (type === 'text' && element.editable && !element.lock) {
                this.showTextEditor(element);
            } else if (type === 'image') {
                this.showImageCroper(element);
            } else if (type === 'mask') {
                this.showMaskEditor(element);
            }
        },
        showTextEditor: function showTextEditor(element) {
            element.$editing = true;
        },
        replaceImage: function replaceImage(url, options, element) {
            element = this.getElement(element);
            if (!element) {
                return;
            }

            var type = element.type;
            if (type !== 'image' && type !== 'mask') {
                return;
            }

            options = (0, _assign3.default)({
                forwardEdit: true
            }, options);

            var elementWidth = element.width;
            var elementHeight = element.height;
            var imgWidth = options.width || elementWidth;
            var imgHeight = options.height || elementHeight;

            var ratio = Math.max(elementWidth / imgWidth, elementHeight / imgHeight);
            var width = imgWidth * ratio;
            var height = imgHeight * ratio;

            var xOffset = width - elementWidth;
            var yOffset = height - elementHeight;

            if (type === 'image') {
                if (!element.clip) {
                    this.$set(element, 'clip', { left: 0, right: 0, bottom: 0, top: 0 });
                }
                this.changeElement({
                    url: url,
                    clip: {
                        left: xOffset / 2,
                        right: xOffset / 2,
                        bottom: yOffset / 2,
                        top: yOffset / 2
                    }
                }, element);
            } else if (type === 'mask') {
                this.changeElement({
                    url: url,
                    imageWidth: width,
                    imageHeight: height,
                    imageTransform: {
                        rotation: 0,
                        position: {
                            x: -xOffset / 2,
                            y: -yOffset / 2
                        }
                    }
                }, element);
            }

            if (options.forwardEdit) {
                if (type === 'image') {
                    this.showImageCroper(element);
                } else if (type === 'mask') {
                    this.showMaskEditor(element);
                }
            }
        },
        pickImage: function pickImage(callback, afterSelect) {
            var _this3 = this;

            return (0, _pickImage3.default)().then(function (file) {
                if (afterSelect) {
                    afterSelect(file);
                }

                return _loader2.default.readFile(file, 'DataURL');
            }).then(function (url) {
                return _loader2.default.loadImage(url);
            }).then(function (img) {
                callback.call(_this3, img, img.src);

                return img;
            }).catch(function (err) {
                console.error('Image file load error:', err.stack);
            });
        },
        showImageCroper: function showImageCroper(element) {
            element = this.getElement(element);

            this.currentCropElement = element;
        },
        hideImageCroper: function hideImageCroper() {
            this.currentCropElement = null;
        },
        showMaskEditor: function showMaskEditor(element) {
            element = this.getElement(element);

            this.currentEditMask = element;
        },
        hideMaskEditor: function hideMaskEditor() {
            this.currentEditMask = null;
        },
        toggleCropperFocus: function toggleCropperFocus() {
            var currentElement = this.currentElement;

            if (!currentElement || currentElement.type !== '$masker' && currentElement.type !== '$croper') {
                return;
            }

            var isInnerActive = currentElement.resize === 7;
            var name = isInnerActive ? 'imageCroper.activeOuter' : 'imageCroper.activeInner';

            this.$events.$emit(name);
        },
        initRect: function initRect() {
            this.updateRectThrottle = (0, _throttle3.default)(this.updateContainerRect, 100);

            this.$watch('zoomWidthHeight', this.updateRectThrottle);

            this.$events.$on('editor.resize', this.updateRectThrottle);
            this.$events.$on('editor.scroll', this.updateRectThrottle);

            this.updateRectThrottle();
        },
        updateContainerRect: function updateContainerRect() {
            var mode = this.mode;
            var container = this.container;
            var shellHeight = this.height * this.zoom;
            var boundingRect = container[0].getBoundingClientRect();
            var containerRect = this.containerRect;
            var padding = this.options.padding;

            containerRect.padding = padding;

            containerRect.scrollLeft = container.prop('scrollLeft');
            containerRect.scrollTop = container.prop('scrollTop');

            containerRect.scrollWidth = container[0].scrollWidth;
            containerRect.scrollHeight = container[0].scrollHeight;

            containerRect.height = boundingRect.height;
            containerRect.width = boundingRect.width;

            containerRect.left = boundingRect.left;
            containerRect.top = boundingRect.top;

            var offset = 0;
            if (mode !== 'mirror') {
                offset = boundingRect.height - shellHeight - 2 * padding;
                offset = Math.max(0, offset);
            }

            containerRect.topOffset = offset / 2;

            this.$nextTick(this.updateShellRect);
        },
        updateShellRect: function updateShellRect() {
            var shellRect = this.shellRect;
            var containerRect = this.containerRect;
            var shell = (0, _jquery2.default)('.editor-shell', this.$el);

            if (shell.length) {
                var rect = shell[0].getBoundingClientRect();

                (0, _assign3.default)(shellRect, {
                    height: rect.height,
                    width: rect.width,
                    top: rect.top - containerRect.top,
                    left: rect.left - containerRect.left
                });
            }

            return shellRect;
        },
        pointFromEvent: function pointFromEvent(e) {
            var zoom = this.zoom;
            var shellRect = this.shellRect;
            var containerRect = this.containerRect;
            var xOffset = containerRect.left + shellRect.left;
            var yOffset = containerRect.top + shellRect.top;

            return {
                x: (e.pageX - xOffset) / zoom,
                y: (e.pageY - yOffset) / zoom
            };
        },
        initDomEvents: function initDomEvents() {
            var _this4 = this;

            var self = this;
            var doc = (0, _jquery2.default)(document);
            var container = this.container;

            this.hotkeys = new _hotkeys.Hotkeys();
            this.handleMap = _hotkeys.handleMap;
            var bindHotkey = this.innerProps.options.bindHotkey;

            var bindDefault = this.mode === 'design' || this.mode === 'flow';

            if (bindHotkey === true || bindHotkey === null && bindDefault) {
                this.enableHotkey();
            }

            doc.on('keydown.editor', function (e) {
                if (_this4.currentElement && _this4.currentElement.lock) {
                    return;
                }
                self.$events.$emit('editor.keydown', e);

                if (e.isDefaultPrevented() || _utils2.default.isEditable(e.target)) {
                    return;
                }

                if (self.editable && _this4.hotkeys && _this4.hotkeys.fire(e) === false) {
                    e.preventDefault();
                }
            });

            function addFromImageDom(imageDom) {
                var layout = this.currentLayout;
                if (layout) {
                    var ratio = Math.max(imageDom.width / layout.width, imageDom.height / layout.height, 1);

                    var imageElement = this.addImage(imageDom.src, {
                        width: imageDom.width / ratio,
                        height: imageDom.height / ratio
                    });

                    this.focusElement(imageElement);
                }
            }

            void new _pasteImage2.default(addFromImageDom.bind(this), this.pasteElement);

            void new _dropFile2.default(document.body, {
                dragover: function dragover(e) {
                    return e.preventDefault();
                },
                drop: addFromImageDom.bind(this)
            });

            function editorEvent(e) {
                var target = e.target;
                var evtName = 'editor.' + e.type;

                if (target !== container[0] && !_jquery2.default.contains(container[0], target)) {
                    evtName += '.out';
                }

                self.$events.$emit(evtName, e);

                if (!e.isDefaultPrevented()) {
                    self.$events.$emit(evtName + '.core', e);
                }
            }

            doc.on('mousedown.editor click.editor', editorEvent);

            (0, _jquery2.default)(this.$el).on('contextmenu', function (e) {
                e.preventDefault();

                self.$events.$emit('editor.contextmenu', e);
            });

            this.$on('destroy', function () {
                doc.off('.editor', editorEvent);
                doc = null;
            });
        },
        updateCurrentSelection: function updateCurrentSelection() {
            var _this5 = this;

            var textSelectionKeys = Object.keys(_config.commandValueMap);

            var textDefaultValue = {
                fontWeight: 400,
                fontStyle: 'normal',
                textDecoration: 'none'
            };

            var textSelection = {};
            textSelectionKeys.forEach(function (key) {
                textSelection[key] = null;
            });

            textSelectionKeys.forEach(function (key) {
                textSelection[key + 'Mixed'] = false;
            });

            var clearSelectionWatch = function clearSelectionWatch() {};

            var diffContentsProps = function diffContentsProps(elements) {
                var result = {};
                var contents = [];

                elements.forEach(function (element) {
                    var child = element.contents.map(function (item) {
                        return _extends({}, item, {
                            element: element
                        });
                    });

                    contents = contents.concat(child);
                });

                contents.length > 1 && contents.reduce(function (a, b) {
                    textSelectionKeys.forEach(function (key) {
                        var aValue = a[key] || a.element[key] || textDefaultValue[key];
                        var bValue = b[key] || b.element[key] || textDefaultValue[key];

                        if (key === 'fontSize') {
                            aValue = Math.round(aValue);
                            bValue = Math.round(bValue);
                        };

                        if (aValue !== bValue) {
                            result[key + 'Mixed'] = true;
                        }
                    });

                    return b;
                });

                return result;
            };

            var getElementSelection = function getElementSelection(element) {
                var props = {};

                textSelectionKeys.forEach(function (key) {
                    props[key] = element[key] || textDefaultValue[key];
                });

                props.fontSize = Math.round(props.fontSize);

                return props;
            };

            this.$watch(function () {
                var result = null;
                var currentElement = _this5.currentElement,
                    currentSubElement = _this5.currentSubElement,
                    selectedElements = _this5.selectedElements;

                var elements = selectedElements || [];
                currentElement = currentSubElement || currentElement;

                if (currentElement && currentElement.type === 'text' && !currentElement.$editing) {
                    result = _extends({}, diffContentsProps([currentElement]), getElementSelection(currentElement));
                }

                if (currentElement && currentElement.type === 'group') {
                    elements = currentElement.elements;
                } else if (elements.length > 1) {
                        elements = elements.reduce(function (a, b) {
                            return a.concat(b.elements || b);
                        }, []);
                    }

                elements = elements.filter(function (element) {
                    return element.type === 'text';
                });

                if (elements.length) {
                    result = _extends({}, diffContentsProps(elements), getElementSelection(elements[0]));
                }

                return {
                    result: result,
                    length: elements.length,
                    currentElement: currentElement
                };
            }, function (_ref) {
                var result = _ref.result,
                    currentElement = _ref.currentElement;

                if (result) {
                    _this5.currentTextSelection = Object.assign({}, textSelection, result);
                } else if (!length && (!currentElement || currentElement.type !== 'text')) {
                    _this5.currentTextSelection = null;
                }
            });

            this.$watch('currentRichText', function (richText) {
                var currentElement = _this5.currentElement,
                    currentSubElement = _this5.currentSubElement;

                currentElement = currentSubElement || currentElement;

                clearSelectionWatch();

                if (!currentElement || !richText) {
                    return;
                }

                var callback = function callback() {
                    var currentElement = _this5.currentElement,
                        currentSubElement = _this5.currentSubElement;

                    currentElement = currentSubElement || currentElement;

                    if (!currentElement.$editing) return;

                    var props = {};

                    textSelectionKeys.forEach(function (key) {
                        var val = richText.cmd.queryCommandValue(key);
                        props[key] = _config.commandValueMap[key](val) || currentElement[key];
                    });

                    props.fontSize = Math.round(props.fontSize / _this5.zoom);
                    _this5.currentTextSelection = Object.assign({}, textSelection, props);

                    var range = richText.selection._currentRange;
                    if (!range) return;

                    var selectedNodes = (0, _jquery2.default)('<div></div>').append(range.cloneContents());
                    if (!selectedNodes.children().length) return;

                    var contents = _utils.serialize.fromJSON(selectedNodes.html());
                    if (!contents || contents.length < 2) return;

                    var result = diffContentsProps([{ contents: contents }]);
                    Object.assign(_this5.currentTextSelection, result);
                };

                richText.on('change', callback);
                richText.selection.on('change', callback);

                clearSelectionWatch = function clearSelectionWatch() {
                    richText.off('change', callback);
                    richText.selection.off('change', callback);
                };
            });
        }
    },
    events: {
        'editor.templet.ready': function editorTempletReady() {
            if (this.options.autoFitZoom) {
                this.fitZoom();
            }
        },
        'editor.resize': function editorResize() {
            clearTimeout(this.resizeFitZoomTimer);

            if (this.options.autoFitZoom) {
                this.resizeFitZoomTimer = setTimeout(this.fitZoom, 120);
            }
        },
        'editor.zoom': function editorZoom() {},
        'editor.click.out.core': function editorClickOutCore() {},
        'editor.click.core': function editorClickCore(e) {
            if (_utils2.default.isEditable(e.target)) {
                return;
            }

            var element = void 0;
            var point = this.pointFromEvent(e);

            var isMac = navigator.platform.indexOf('Mac') > -1;

            if (e.shiftKey || (isMac ? e.metaKey : e.ctrlKey)) {
                element = this.getElementByPoint(point.x, point.y);
                if (element) {
                    var currentElement = this.currentElement;

                    if (currentElement && currentElement.lock) {
                        currentElement = null;
                    }

                    if (element.$selected) {
                        this.unselectElement(element);
                    } else {
                        !element.lock && this.selectElement(element);
                    }

                    if (element === currentElement) {
                        this.unselectElement(element);
                        if (!this.selectedElements.length) {
                            this.currentElement = null;
                        }
                    } else {
                        this.selectElement(currentElement);
                    }
                }
            } else {
                element = this.focusElementByPoint(point.x, point.y);
                if (!element) {
                    this.currentElement = null;
                }
            }
        },
        'editor.transform.resizing': function editorTransformResizing(drag, element) {
            var dir = drag.dir;
            var preWidth = drag.width;
            var preHeight = drag.height;
            var preLeft = drag.left;
            var preTop = drag.top;

            if (dir.indexOf('w') > -1) {
                drag.dx *= -1;
            }
            if (dir.indexOf('n') > -1) {
                drag.dy *= -1;
            }

            var width = Number(preWidth) + drag.dx;
            var height = Number(preHeight) + drag.dy;

            var widthTo = width;
            var heightTo = height;

            var limit = void 0;
            if (element.$getResizeLimit) {
                limit = element.$getResizeLimit(drag);

                width = Math.max(limit.minWidth, width);
                height = Math.max(limit.minHeight, height);

                width = Math.min(limit.maxWidth, width);
                height = Math.min(limit.maxHeight, height);
            }

            var minWidth = 0;
            var minHeight = 0;
            if (element.type === 'text' && element.autoScale) {
                if (dir.length === 1) {
                    minWidth = element.fontSize;
                    minHeight = element.fontSize * element.lineHeight;
                } else {
                    minWidth = minHeight = 10;
                }
            } else if (element.type === 'group' && element.autoGrow) {
                    var svgChild = element.elements.find(function (el) {
                        return el.type === 'svg';
                    });
                    if (svgChild) {
                        minWidth = svgChild.$minWidth;
                        minHeight = svgChild.$minHeight;
                    }
                }

            width = Math.max(minWidth, width);
            height = Math.max(minHeight, height);

            if (limit && dir.length > 1) {
                if (width !== widthTo) {
                    height = Math.max(limit.minHeight, heightTo * width / widthTo);
                }

                if (height !== heightTo) {
                    width = widthTo * height / heightTo;
                }
            }

            element.width = width;
            element.height = height;

            (function () {
                var points = _utils2.default.getRectPoints({
                    left: preLeft,
                    top: preTop,
                    width: width,
                    height: height,
                    rotate: element.rotate,
                    skewX: element.skewX,
                    skewY: element.skewY
                });

                var diagonal = {
                    nw: 'se',
                    ne: 'sw',
                    n: 'sw',
                    w: 'se',
                    sw: 'ne',
                    se: 'nw',
                    s: 'ne',
                    e: 'nw'
                }[dir];

                var newPoint = points[diagonal];
                var prePoint = drag.points[diagonal];

                element.left = preLeft - (newPoint.x - prePoint.x);
                element.top = preTop - (newPoint.y - prePoint.y);
            })();

            var startClip = drag.clip;
            if (startClip) {
                var clip = element.clip;
                var ratioX = width / preWidth / this.zoom;
                var ratioY = height / preHeight / this.zoom;

                clip.left = startClip.left * ratioX;
                clip.top = startClip.top * ratioY;
                clip.right = startClip.right * ratioX;
                clip.bottom = startClip.bottom * ratioY;
            }
        },
        'editor.transform.rotating': function editorTransformRotating(drag, element) {
            element.rotate = drag.rotate;
        },
        'editor.transform.inactive': function editorTransformInactive(element) {
            this.makeSnapshotByElement(element);
        },
        'editor.image.picker.show': function editorImagePickerShow(hook) {
            var hookImagePicker = this.options.hookImagePicker;

            if (hookImagePicker) {
                hook();
                this.$events.$emit('editor.element.picker');
            }

            return hookImagePicker;
        },
        'editor.croper.update': function editorCroperUpdate(val) {},
        'editor.element.picker': function editorElementPicker() {
            this.$emit('editor-element-picker');
        },
        'editor.currentRichText.update': function editorCurrentRichTextUpdate(richText) {
            this.currentRichText = richText;
        }
    },

    watch: {},

    mounted: function mounted() {
        var _this6 = this;

        var self = this;

        var container = this.container = (0, _jquery2.default)(this.$el);
        this.initRect();

        this.initDomEvents();

        var lazyEventHandle = function lazyEventHandle(e) {
            var type = e.type;

            clearTimeout(lazyEventHandle['timer_' + type]);
            lazyEventHandle['timer_' + type] = setTimeout(function () {
                self.$events.$emit('editor.' + type, e);
            }, 64);
        };

        container.on('scroll.editor', lazyEventHandle);
        view.on('resize.editor', lazyEventHandle);

        this.$on('destroy', function () {
            view.off('resize.editor', lazyEventHandle);
            container.off('scroll.editor', lazyEventHandle);

            _this6.container = null;
        });

        var events = this.$options.events;
        for (var key in events) {
            this.$on(key, events[key]);
        }

        this.$on('editor.templet.ready', function () {
            _this6.$events.$emit('editor.templet.ready');
        });

        if ('') {
            this.utils = _utils2.default;
            window.editor = this;

            window.editor.debug = function () {
                return _this6.setTemplet({}).then(function () {
                    var element = _this6.addImage('//172.16.2.198/d/18890@0e_0o_1l_800h_800w.png', {
                        resize: 7,
                        height: 150,
                        width: 200,
                        left: 360,
                        top: 80
                    });
                });
            };
        }

        this.updateCurrentSelection();
    },
    beforeDestroy: function beforeDestroy() {
        this.$events.$off('editor.resize', this.updateRectThrottle);
        this.$events.$off('editor.scroll', this.updateRectThrottle);

        this.handleMap = null;
        this.hotkeys = null;
    }
});
module.exports = exports['default'];