'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _cloneDeep2 = require('lodash/cloneDeep');

var _cloneDeep3 = _interopRequireDefault(_cloneDeep2);

var _merge2 = require('lodash/merge');

var _merge3 = _interopRequireDefault(_merge2);

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _matrix = require('../utils/matrix');

var _matrix2 = _interopRequireDefault(_matrix);

var _transform = require('../utils/transform');

var _transform2 = _interopRequireDefault(_transform);

var _utils = require('../utils/utils');

var _utils2 = _interopRequireDefault(_utils);

var _elementModel = require('../utils/element-model');

var _editorDefaults = require('../editor-defaults');

var _editorDefaults2 = _interopRequireDefault(_editorDefaults);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var ElementModel = function () {
    function ElementModel(data) {
        _classCallCheck(this, ElementModel);

        var type = data && data.type;

        if (!type) {
            throw new Error('No type to create model: ' + type);
        }

        data.transform = this.parseTransform(data.transform);

        (0, _merge3.default)(this, _editorDefaults2.default.element, _editorDefaults2.default.elementExts, _editorDefaults2.default[type + 'Element'], data);

        if (this.boxShadow) {
            var shadow = this.boxShadow;

            shadow.color = (0, _elementModel.hexaToRgba)(shadow.color);
        }

        if (!this.$id) {
            this.$id = _utils2.default.uuid();
        }

        this.version = _editorDefaults2.default.version;
    }

    _createClass(ElementModel, [{
        key: 'parseTransform',
        value: function parseTransform(matrixData) {
            var transform = new _transform2.default();

            if (matrixData) {
                if (matrixData.localTransform) {
                    transform = (0, _cloneDeep3.default)(matrixData);
                } else {
                    var matrix = new _matrix2.default();
                    matrix.copy.call(matrixData, matrix);
                    matrix.decompose(transform);
                }
            }
            return transform;
        }
    }, {
        key: 'reset',
        value: function reset() {
            var id = this.$id;
            var extProps = (0, _cloneDeep3.default)(_editorDefaults2.default.elementExts);

            (0, _merge3.default)(this, extProps);

            this.$id = id;
        }
    }, {
        key: 'enableEditable',
        value: function enableEditable() {
            this.rotatable = true;
            this.dragable = true;
        }
    }, {
        key: 'disableEditable',
        value: function disableEditable() {
            this.rotatable = false;
            this.dragable = false;
        }
    }, {
        key: 'toString',
        value: function toString() {
            return JSON.stringify(this);
        }
    }, {
        key: 'rotation',
        get: function get() {
            return this.transform.rotation;
        },
        set: function set(rotation) {
            this.transform.rotation = rotation;
        }
    }, {
        key: 'rotate',
        get: function get() {
            return _utils2.default.radToDeg(this.rotation);
        },
        set: function set(deg) {
            this.transform.rotation = _utils2.default.degToRad(deg);
        }
    }, {
        key: 'scaleX',
        get: function get() {
            return this.transform.scale.x;
        },
        set: function set(v) {
            this.transform.scale.x = v;
        }
    }, {
        key: 'scaleY',
        get: function get() {
            return this.transform.scale.y;
        },
        set: function set(v) {
            this.transform.scale.y = v;
        }
    }, {
        key: 'skewX',
        get: function get() {
            return this.transform.skew.x;
        },
        set: function set(v) {
            this.transform.skew.x = v;
        }
    }, {
        key: 'skewY',
        get: function get() {
            return this.transform.skew.y;
        },
        set: function set(v) {
            this.transform.skew.y = v;
        }
    }]);

    return ElementModel;
}();

exports.default = ElementModel;
module.exports = exports['default'];