'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _defaultsDeep2 = require('lodash/defaultsDeep');

var _defaultsDeep3 = _interopRequireDefault(_defaultsDeep2);

var _utils = require('../utils/utils');

var _elementModel = require('../utils/element-model');

var _editorDefaults = require('../editor-defaults');

var _editorDefaults2 = _interopRequireDefault(_editorDefaults);

var _elementBaseModel = require('./element-base-model');

var _elementBaseModel2 = _interopRequireDefault(_elementBaseModel);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ElementTextModel = function (_ElementBaseModel) {
    _inherits(ElementTextModel, _ElementBaseModel);

    function ElementTextModel(data) {
        _classCallCheck(this, ElementTextModel);

        var modelVersion = _utils.version.parse(data.version || '0.0.0');

        if (modelVersion.major < 2) {
            var mode = {
                'lr-tb': 'horizontal-tb',
                'tb-rl': 'vertical-rl'
            };

            if (mode[data.writingMode]) {
                data.writingMode = mode[data.writingMode];
            }

            var padding = data.padding || _editorDefaults2.default.textElement.padding;
            data.width += padding[1] + padding[3];
            data.height += padding[0] + padding[2];
        }

        if (modelVersion.number >= 3002000) {
            data.textEffects.forEach(function (effect) {
                (0, _defaultsDeep3.default)(effect, _editorDefaults2.default.textEffect);
            });
        }

        var _this = _possibleConstructorReturn(this, (ElementTextModel.__proto__ || Object.getPrototypeOf(ElementTextModel)).call(this, data));

        if (_this.backgroundColor) {
            _this.backgroundColor = (0, _elementModel.hexaToRgba)(_this.backgroundColor);
        }

        if (_this.color) {
            _this.color = (0, _elementModel.hexaToRgba)(_this.color);
        }

        if (_this.textShadow) {
            var shadow = _this.textShadow;

            shadow.color = (0, _elementModel.hexaToRgba)(shadow.color);
        }

        (0, _elementModel.useEffects)('text', _this);

        if (_this.textEffects && _this.textEffects.length) {
            _this.textEffects.forEach(function (effect) {
                if (effect.stroke) {
                    effect.stroke.color = (0, _elementModel.hexaToRgba)(effect.stroke.color);
                }

                if (effect.shadow) {
                    effect.shadow.color = (0, _elementModel.hexaToRgba)(effect.shadow.color);
                }

                if (effect.filling) {
                    effect.filling.color = (0, _elementModel.hexaToRgba)(effect.filling.color);

                    if (effect.filling.gradient) {
                        effect.filling.gradient.stops.forEach(function (stopPoint) {
                            stopPoint.color = (0, _elementModel.hexaToRgba)(stopPoint.color);
                        });
                    }
                }
            });
        }

        if (!_this.contents || !_this.contents.length) {
            _this.contents = _utils.serialize.fromJSON(_this.content || '', _this);
        }

        var autoScale = data.autoScale,
            _data$writingMode = data.writingMode,
            writingMode = _data$writingMode === undefined ? 'horizontal-tb' : _data$writingMode,
            resize = data.resize;


        if (autoScale === undefined && (!resize || writingMode === 'horizontal-tb' && resize === 5 || writingMode === 'vertical-rl' && resize === 3)) {
            _this.autoScale = true;
        }
        return _this;
    }

    return ElementTextModel;
}(_elementBaseModel2.default);

exports.default = ElementTextModel;
module.exports = exports['default'];