'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _assign2 = require('lodash/assign');

var _assign3 = _interopRequireDefault(_assign2);

var _throttle2 = require('lodash/throttle');

var _throttle3 = _interopRequireDefault(_throttle2);

var _jquery = require('jquery');

var _jquery2 = _interopRequireDefault(_jquery);

var _utils = require('../utils/utils');

var _utils2 = _interopRequireDefault(_utils);

var _editorToolbarPop = require('./editor-toolbar-pop.html');

var _editorToolbarPop2 = _interopRequireDefault(_editorToolbarPop);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = {
    props: ['options', 'global', 'currentLayout', 'model', 'subModel'],
    template: _editorToolbarPop2.default,
    data: function data() {
        return {
            clickLocked: false,
            rect: {
                height: 0,
                width: 0,
                left: -99999,
                top: -99999
            },
            toolbarPosition: 'toolbar-north'
        };
    },

    computed: {
        editor: function editor() {
            return this.$parent;
        },
        type: function type() {
            return this.model.type.replace('$', '');
        },
        show: function show() {
            var model = this.model;
            var global = this.global;

            if (!model || model.$draging || global.$draging) {
                this.rect.left = -99999;
                return false;
            }

            return true;
        },
        isRenderToolbar: function isRenderToolbar() {
            if (this.type === 'croper' || this.type === 'masker') {
                return this.options.showEditConfirmToolbar;
            }

            return this.options.showToolbar;
        }
    },
    methods: {
        updateRect: (0, _throttle3.default)(function () {
            var global = this.global;
            var editor = this.editor;

            var model = this.model;
            var popShell = (0, _jquery2.default)(this.$el);

            if (!model || !popShell.length || !this.isRenderToolbar) {
                return;
            }

            var editorShellRect = editor.shellRect;
            var editorContainerRect = editor.containerRect;
            var popShellRect = popShell[0].getBoundingClientRect();

            var zoom = global.zoom;
            var bbox = _utils2.default.getBBoxByElement(model, zoom);
            var top = bbox.top - popShellRect.height + this.currentLayout.top * zoom;
            var left = bbox.left;

            var followOffset = 12;

            if (!this.$refs.toolbar) {
                return false;
            }

            var neededSpace = (this.$refs.toolbar.neededSpace || 0) + popShellRect.height + followOffset;

            var transformRect = editor.$refs.transform.getBoundingClientRect();
            var canvasRect = editor.$el.getBoundingClientRect();

            var topSpace = transformRect.top - canvasRect.top;
            var bottomSpace = canvasRect.bottom - transformRect.bottom;

            var relative = null;
            if (topSpace > neededSpace) {
                relative = 'top';
                this.toolbarPosition = 'toolbar-north';
            } else if (bottomSpace > neededSpace) {
                relative = 'bottom';
                this.toolbarPosition = 'toolbar-south';
                top = bbox.top + bbox.height + this.currentLayout.top * zoom;
                followOffset = -12;
            } else if (bottomSpace > topSpace) {
                relative = 'bottom';
                this.toolbarPosition = 'toolbar-north';
                top = bbox.top + bbox.height + this.currentLayout.top * zoom;
                followOffset = -12;
            } else {
                relative = 'top';
                this.toolbarPosition = 'toolbar-south';
            }

            var topOffset = transformRect._transformEleRect.top - transformRect._rotatorHandleRect.top;
            var bottomOffset = transformRect._transformEleRect.bottom - transformRect._rotatorHandleRect.bottom;

            top -= followOffset;

            if (topOffset > 0 && relative === 'top') {
                top -= topOffset;
            } else if (bottomOffset < 0 && relative === 'bottom') {
                top -= bottomOffset;
            }

            var maxLeft = editorContainerRect.width - editorShellRect.left - popShellRect.width;
            var maxTop = editorContainerRect.height - editorShellRect.top - popShellRect.height;
            var minLeft = -editorShellRect.left;
            var minTop = -editorShellRect.top;

            var positionOffset = 8;
            minLeft += positionOffset;
            minTop += positionOffset;
            maxLeft -= positionOffset;
            maxTop -= positionOffset;

            left = Math.max(minLeft, Math.min(maxLeft, left));
            top = Math.max(minTop, Math.min(maxTop, top));

            (0, _assign3.default)(this.rect, {
                height: popShellRect.height,
                width: popShellRect.width,
                left: left,
                top: top
            });
        }, 64, { leading: false }),

        autoUpdateRect: function autoUpdateRect() {
            var watcher = function watcher() {
                if (!this.show) {
                    return;
                }
                this.updateRect();
            };

            this.$watch('global.zoom', watcher);
            this.$watch('subModel', watcher);
            this.$watch('model', watcher);
            this.$watch('show', watcher);

            this.updateRect();
        },
        onMousedown: function onMousedown() {
            this.clickLocked = true;
        },
        onClick: function onClick() {
            this.clickLocked = true;
        }
    },
    events: {
        'editor.click': function editorClick(e) {
            if (this.clickLocked && !_utils2.default.isEditable(e.target)) {
                e.preventDefault();
            }

            this.clickLocked = false;

            return true;
        },
        'editor.resize': function editorResize() {
            this.updateRect();

            return true;
        },
        'editor.scroll': function editorScroll() {
            this.updateRect();

            return true;
        }
    },
    mounted: function mounted() {
        this.autoUpdateRect();
    }
};
module.exports = exports['default'];