'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _editorToolbarConfig = require('./editor-toolbar-config.js');

var _editorToolbarConfig2 = _interopRequireDefault(_editorToolbarConfig);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

exports.default = {
    props: ['options', 'global', 'model', 'subModel', 'disableLayerEdit', 'template'],

    computed: {
        editor: function editor() {
            return this.$parent.editor;
        }
    },
    methods: {
        set: function set(name, val, element) {
            var props = {};

            if ((typeof name === 'undefined' ? 'undefined' : _typeof(name)) === 'object') {
                props = name;
                element = val;
            } else {
                props = _defineProperty({}, name, val);
            }

            element = element || this.model;

            for (var k in props) {
                if (element[k] === props[k]) {
                    delete props[k];
                }
            }

            Object.keys(props).length && this.editor.changeElement(props, element);
        },
        remove: function remove() {
            this.editor.removeElement();
        },
        clone: function clone() {
            this.editor.cloneElement();
        },
        lock: function lock() {
            this.editor.lockElement();
        },
        unlock: function unlock() {
            this.editor.unlockElement();
        },
        setToTop: function setToTop() {
            this.editor.setElementToTop();
        },
        setToBottom: function setToBottom() {
            this.editor.setElementToBottom();
        },
        flip: function flip(dir) {
            this.editor.flipElement(dir, this.model);
        },
        setOpacity: function setOpacity(opacity) {
            this.set('opacity', opacity);
        },
        replaceImage: function replaceImage(url, width, height) {
            this.editor.replaceImage(url, {
                width: width,
                height: height
            }, this.model);
        },
        showLoadingTip: function showLoadingTip() {
            this.model.$loaded = false;
        },
        showEditor: function showEditor() {
            var model = this.model;

            this.editor.showElementEditor(model);
        },
        applyEdit: function applyEdit() {
            this.$events.$emit('editor.edit.apply', this.model);
        },
        cancelEdit: function cancelEdit() {
            this.$events.$emit('editor.edit.cancel', this.model);
        }
    },

    created: function created() {
        if (this.template) {
            if (typeof this.template === 'string') {
                this.$options.template = _editorToolbarConfig2.default.compile(this.template);
            } else {
                this.$options.template = _editorToolbarConfig2.default.compile(this.template.content || '');
                if (this.template.neededSpace !== undefined) {
                    this.neededSpace = this.template.neededSpace;
                };
            }
        }
    }
};
module.exports = exports['default'];