'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _resizeImage = require('./resize-image');

var _resizeImage2 = _interopRequireDefault(_resizeImage);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var sin = Math.sin,
    cos = Math.cos,
    atan = Math.atan,
    sqrt = Math.sqrt,
    round = Math.round;

var createCanvas = function createCanvas() {
    var width = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 100;
    var height = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 100;

    var canvas = document.createElement('canvas');

    canvas.height = round(height);
    canvas.width = round(width);

    return canvas;
};

var canvasTools = {
    createCanvas: createCanvas,

    resizeImageByCanvas: function resizeImageByCanvas(image) {
        var width = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 100;
        var height = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 100;
        var resizeOptions = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};

        var canvas = createCanvas(width, height);

        return (0, _resizeImage2.default)(image, canvas, resizeOptions);
    },
    drawImageToCanvas: function drawImageToCanvas(canvas, image, imageCenter, imageSize, imageRotation) {
        var blendMode = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : 'source-in';

        var context = canvas.getContext('2d');

        context.globalCompositeOperation = blendMode;

        var imageRadius = sqrt(imageSize.width * imageSize.width + imageSize.height * imageSize.height) / 2;
        var turn = atan(imageSize.height / imageSize.width);

        var x = round(imageCenter.x - cos(imageRotation + turn) * imageRadius);
        var y = round(imageCenter.y - sin(imageRotation + turn) * imageRadius);

        context.translate(x, y);
        context.rotate(imageRotation);
        context.drawImage(image, 0, 0, imageSize.width, imageSize.height);

        context.setTransform(1, 0, 0, 1, 0, 0);

        return canvas;
    }
};

exports.default = canvasTools;
module.exports = exports['default'];