'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = escape;


var r = /^<\/span>/i;
var rSpanTagOpen = /^<span[^>]*>/i;
var rTagClose = /^<\/[\w]+>/i;
var rTagOpen = /^<[^<]+>/i;
var rValue = /^[^<]+/i;

var sanitize = function sanitize(html) {
    return html.replace(/</g, '&lt;').replace(/>/g, '&gt;');
};
var noop = function noop(str) {
    return str;
};

var rules = [{ match: function match(str) {
        return r.exec(str);
    }, convert: noop }, { match: function match(str) {
        return rSpanTagOpen.exec(str);
    }, convert: noop }, { match: function match(str) {
        return rTagClose.exec(str);
    }, convert: sanitize }, { match: function match(str) {
        return rTagOpen.exec(str);
    }, convert: sanitize }, {
    match: function match(str) {
        var result = '';
        var value = rValue.exec(str);
        if (!value) return null;

        while (true) {
            var _value = rValue.exec(str);
            if (_value) {
                result += _value[0];
                str = str.substr(_value[0].length);
            } else if (str[0] === '<' && !rTagOpen.exec(str)) {
                str = str.substr(1);
                result += '<';
            } else {
                break;
            }
        }

        return [result];
    },

    convert: sanitize
}];

function getToken(str) {
    for (var i = 0; i < rules.length; i++) {
        var raw = rules[i].match(str);
        if (raw) {
            return {
                len: raw[0].length,
                sanitized: rules[i].convert(raw[0])
            };
        }
    }
}

function escape(str) {
    var keepSpan = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

    if (!keepSpan) {
        return sanitize(str);
    }

    var results = '';

    while (str.length > 0) {
        var token = getToken(str);
        str = str.substr(token.len);
        results += token.sanitized;
    }
    return results;
}
module.exports = exports['default'];