'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _keys2 = require('lodash/keys');

var _keys3 = _interopRequireDefault(_keys2);

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _fabric = require('fabric');

var _bluebird = require('bluebird');

var _creator = require('./utils/creator');

var _creator2 = _interopRequireDefault(_creator);

var _clone = require('./utils/clone');

var _masks = require('./data/masks');

var _masks2 = _interopRequireDefault(_masks);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var EffectMaker = function () {
    function EffectMaker(img, zoom) {
        _classCallCheck(this, EffectMaker);

        var image = new _fabric.fabric.Image(img);

        this.zoom = zoom;
        this.image = image;
        this.group = new _fabric.fabric.Group([this.image]);

        this.addObject = this.addObject.bind(this);
    }

    _createClass(EffectMaker, [{
        key: 'setOffset',
        value: function setOffset(options) {
            this.group.set({
                left: options.x === undefined ? this.group.left : options.x,
                top: options.y === undefined ? this.group.top : options.y
            });

            return _bluebird.Promise.resolve(this.group);
        }
    }, {
        key: 'setShadow',
        value: function setShadow(options) {
            var _this = this;

            var defaultOptions = {
                blur: 0,
                color: '#000',
                offsetX: 0,
                offsetY: 0
            };

            options = Object.assign(defaultOptions, options);

            return options.blur || options.offsetX || options.offsetY ? _creator2.default.shadow(this.image, options).then(function (shadow) {
                shadow.name = 'shadow';
                _this.addObject(shadow, 0);

                return shadow;
            }) : _bluebird.Promise.resolve();
        }
    }, {
        key: 'setStroke',
        value: function setStroke(options) {
            var _this2 = this;

            var defaultOptions = {
                color: '#000',
                width: 0,
                radius: 0
            };

            options = Object.assign(defaultOptions, options);

            return options.width ? _creator2.default.stroke(this.image, {
                width: options.width,
                color: options.color
            }).then(function (stroke) {
                stroke.name = 'stroke';
                _this2.addObject(stroke, 0);

                return stroke;
            }) : _bluebird.Promise.resolve();
        }
    }, {
        key: 'setFilling',
        value: function setFilling(options) {
            var _this3 = this;

            var type = options.type;


            var handler = {
                gradient: function gradient(options) {
                    var gradient = _creator2.default.gradient(_this3.image, _extends({}, options, {
                        type: options.gradientType
                    }));

                    return (0, _clone.clone)(_this3.image).then(function (mask) {
                        return _creator2.default.maskedObject(gradient, mask);
                    }).then(function (filled) {
                        return new _fabric.fabric.Image(filled, {
                            left: _this3.image.left,
                            top: _this3.image.top
                        });
                    });
                },
                color: function color(options) {
                    return handler.gradient({
                        colorStops: {
                            0: options.color,
                            1: options.color
                        }
                    });
                },
                image: function image(options) {
                    return options.width && options.height && options.image ? _creator2.default.alphaPattern(_this3.image, _extends({}, options, {
                        zoom: _this3.zoom
                    })) : _bluebird.Promise.resolve();
                }
            };

            return handler[type](options).then(function (filling) {
                if (filling) {
                    _this3.addObject(filling);
                    _this3.image.set('visible', false);
                }

                return filling;
            });
        }
    }, {
        key: 'setExpand',
        value: function setExpand(options) {
            var offset = options.offset,
                scale = options.scale;
            var left = offset.left,
                right = offset.right,
                top = offset.top,
                bottom = offset.bottom;

            var image = _creator2.default.expand(this.image, {
                left: left,
                right: right,
                top: top,
                bottom: bottom,
                scale: scale
            }).getElement();

            this.image.setElement(image);

            return _bluebird.Promise.resolve(image);
        }
    }, {
        key: 'setMask',
        value: function setMask(options) {
            var _this4 = this;

            var _options = options,
                type = _options.type;


            if (type !== 'image') {
                options = {
                    image: type === 'custom' ? new _fabric.fabric.Rect({
                        width: this.image.width,
                        height: this.image.height,
                        rx: options.radius,
                        ry: options.radius
                    }) : (0, _masks2.default)(type)
                };
            }

            return type !== 'image' || type === 'image' && options.image ? _creator2.default.image(options.image).then(function (mask) {
                var renderWidth = _this4.image.getScaledWidth();
                var renderHeight = _this4.image.getScaledHeight();

                mask.scale(Math.min(renderWidth / mask.width, renderHeight / mask.height));

                return (0, _clone.clone)(_this4.image).then(function (image) {
                    return _creator2.default.maskedObject(image, mask);
                });
            }).then(function (masked) {
                _this4.image.setElement(masked);
                _this4.image.getElement = function () {
                    return masked;
                };
            }) : _bluebird.Promise.resolve();
        }
    }, {
        key: 'addObject',
        value: function addObject(object, index) {
            if (index === undefined) {
                this.group.add(object);
            } else {
                this.group.insertAt(object, index);
            }

            return object;
        }
    }, {
        key: 'canvas',
        get: function get() {
            return this.group.canvas;
        }
    }]);

    return EffectMaker;
}();

EffectMaker.maskTypes = (0, _keys3.default)(_masks.maskData);
exports.default = EffectMaker;
module.exports = exports['default'];