'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _minBy2 = require('lodash/minBy');

var _minBy3 = _interopRequireDefault(_minBy2);

var _maxBy2 = require('lodash/maxBy');

var _maxBy3 = _interopRequireDefault(_maxBy2);

var _cloneDeep2 = require('lodash/cloneDeep');

var _cloneDeep3 = _interopRequireDefault(_cloneDeep2);

var _map4 = require('lodash/map');

var _map5 = _interopRequireDefault(_map4);

var _every2 = require('lodash/every');

var _every3 = _interopRequireDefault(_every2);

var _isNumber2 = require('lodash/isNumber');

var _isNumber3 = _interopRequireDefault(_isNumber2);

var _isObject2 = require('lodash/isObject');

var _isObject3 = _interopRequireDefault(_isObject2);

var _forEach2 = require('lodash/forEach');

var _forEach3 = _interopRequireDefault(_forEach2);

var _assign2 = require('lodash/assign');

var _assign3 = _interopRequireDefault(_assign2);

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _fabric = require('fabric');

var _bluebird = require('bluebird');

var _sha = require('crypto-js/sha1');

var _sha2 = _interopRequireDefault(_sha);

var _model = require('./model');

var _model2 = _interopRequireDefault(_model);

var _createImage = require('./utils/creator/create-image');

var _createImage2 = _interopRequireDefault(_createImage);

require('./utils/fabric-extend');

var _resizeImage2 = require('../resize-image');

var _resizeImage3 = _interopRequireDefault(_resizeImage2);

var _colorToRgba = require('./utils/color-to-rgba');

var _colorToRgba2 = _interopRequireDefault(_colorToRgba);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var ImageEffect = function () {
    function ImageEffect(canvasEl) {
        _classCallCheck(this, ImageEffect);

        this.isRendering = false;
        this._imgZoom = 1;
        this._lastCall = null;
        this._lastResolve = null;
        this._imageEffectsHash = '';

        this._canvasEl = canvasEl;
        this.canvas = null;
    }

    _createClass(ImageEffect, [{
        key: 'applyEffects',
        value: function applyEffects(img, options) {
            var _this = this;

            var ret = {};
            var _options = options,
                effects = _options.effects,
                lastHash = _options.lastHash,
                clip = _options.clip,
                width = _options.width,
                height = _options.height;

            var url = img.src;

            if (!effects || !effects.length) {
                return _bluebird.Promise.resolve({
                    imageEffectsHash: '',
                    effectedImage: null,
                    effectedHeight: 0,
                    effectedWidth: 0,
                    effectedLeft: 0,
                    effectedTop: 0,
                    originWidth: options.width,
                    originHeight: options.height
                });
            }

            var hashData = { url: url, clip: clip, effects: effects };
            if (this.isImageNeedResize(img, width, height)) {
                (0, _assign3.default)(hashData, {
                    width: width, height: height
                });
            }

            var hashSignStr = JSON.stringify(hashData);
            var currHash = (0, _sha2.default)(hashSignStr);

            var _lastHash = this._imageEffectsHash || lastHash;

            if (_lastHash && '' + _lastHash === '' + currHash && !effects[0].expand.enable) {
                return _bluebird.Promise.resolve(ret);
            }

            this._imageEffectsHash = ret.imageEffectsHash = currHash;

            if (this.isRendering) {
                if (this._lastResolve) {
                    this._lastResolve(ret);
                }

                this._lastCall = function () {
                    return _this.applyEffects(img, options).then(function (ret) {
                        _this._lastCall = null;
                        _this._lastResolve(ret);
                    });
                };

                return new _bluebird.Promise(function (resolve) {
                    _this._lastResolve = resolve;
                });
            }

            this.isRendering = true;
            options = this._processOptions(options);

            return this.resizeImage(img, options).then(function (img) {
                return _this.clipImage(img, options);
            }).then(function (img) {
                return _this.setImage(img);
            }).then(function () {
                return _this.loadEffects(options.effects);
            }).then(function (data) {
                _this.clear();

                _bluebird.Promise.delay(16).then(function () {
                    _this._imageEffectsHash = ret.imageEffectsHash = currHash;
                    _this.isRendering = false;
                    if (_this._lastCall) {
                        _this._lastCall();
                    }
                });

                return (0, _assign3.default)(ret, data);
            });
        }
    }, {
        key: 'resizeImage',
        value: function resizeImage(img, options) {
            var clip = options.clip;


            var ratio = img.width / img.height;
            var width = options.width + clip.left + clip.right;
            var height = options.height + clip.top + clip.bottom;

            if (options.width === Math.max(options.width, options.height)) {
                width = Math.max(width, ImageEffect.resizeThreshold.width);
                height = width / ratio;
            } else {
                height = Math.max(height, ImageEffect.resizeThreshold.height);
                width = height * ratio;
            }

            if (this.isImageNeedResize(img, width, height)) {
                var canvas = new _fabric.fabric.StaticCanvas(null, {
                    width: width,
                    height: height
                });

                this._imgZoom = width / img.width;
                this.resizeEffects(options.effects, width / img.width);

                return (0, _resizeImage3.default)(img, canvas.getElement(), {
                    alpha: true
                });
            }

            return _bluebird.Promise.resolve(img);
        }
    }, {
        key: 'resizeEffects',
        value: function resizeEffects(effects, zoom) {
            (0, _forEach3.default)(effects, function (effect) {
                (0, _forEach3.default)(effect, function (effectItem) {
                    if (!(0, _isObject3.default)(effectItem)) return;

                    (0, _forEach3.default)(effectItem, function (val, key) {
                        if (!(0, _isNumber3.default)(val)) return;

                        effectItem[key] *= zoom;
                    });
                });
            });
        }
    }, {
        key: 'clipImage',
        value: function clipImage(img, _ref) {
            var clip = _ref.clip,
                width = _ref.width,
                height = _ref.height;

            if ((0, _every3.default)(clip, function (val) {
                return !val;
            })) {
                return _bluebird.Promise.resolve(img);
            }

            return (0, _createImage2.default)(img).then(function (image) {
                var ratioX = image.width / (width + clip.left + clip.right);
                var ratioY = image.height / (height + clip.top + clip.bottom);

                var canvas = new _fabric.fabric.StaticCanvas(null, {
                    width: image.width - (clip.right + clip.left) * ratioX,
                    height: image.height - (clip.bottom + clip.top) * ratioY
                });

                image.set({
                    left: -clip.left * ratioX,
                    top: -clip.top * ratioY
                });
                canvas.add(image);
                canvas.renderAll();

                return canvas.getElement();
            });
        }
    }, {
        key: 'setImage',
        value: function setImage(img) {
            this.canvasHeight = img.height;
            this.canvasWidth = img.width;
            this._img = img;
            this.initCanvas(this.canvasWidth, this.canvasHeight);

            return _bluebird.Promise.resolve(img);
        }
    }, {
        key: 'initCanvas',
        value: function initCanvas(width, height) {
            if (!this.canvas) {
                this.canvas = new _fabric.fabric.StaticCanvas(this._canvasEl);
            };
            this.setCanvasSize({
                width: width, height: height
            });
        }
    }, {
        key: 'loadEffects',
        value: function loadEffects(effects) {
            var _this2 = this;

            var offset = this._calcEffectsPosition(effects);

            this._calcCanvasSize(effects);

            return _bluebird.Promise.all((0, _map5.default)(effects, function (effect) {
                return _this2.addEffect(effect);
            })).then(function () {
                var ret = {
                    effectedImage: _this2.toDataURL(),
                    effectedImageHeight: _this2.canvas.height,
                    effectedImageWidth: _this2.canvas.width,
                    effectedImageOffsetLeft: -offset.x,
                    effectedImageOffsetTop: -offset.y,
                    originWidth: _this2._img.width,
                    originHeight: _this2._img.height
                };

                return ret;
            });
        }
    }, {
        key: 'addEffect',
        value: function addEffect(effect) {
            var effectModel = new _model2.default(this);

            this.canvas.insertAt(effectModel.object, 0);

            return effectModel.useData(effect);
        }
    }, {
        key: 'setCanvasSize',
        value: function setCanvasSize(size) {
            var width = Math.floor(size.width);
            var height = Math.floor(size.height);

            this.canvas.setDimensions({
                height: height,
                width: width
            });
        }
    }, {
        key: 'clear',
        value: function clear() {
            this.canvas.clear();
            this._img = null;
        }
    }, {
        key: 'toDataURL',
        value: function toDataURL(options) {
            var dataURL = this.canvas.toDataURL(options);

            return dataURL;
        }
    }, {
        key: 'isImageNeedResize',
        value: function isImageNeedResize(img, width, height) {
            var resizeThreshold = ImageEffect.resizeThreshold;


            return img.width >= width && img.height >= height && img.width >= resizeThreshold.width && img.height >= resizeThreshold.height;
        }
    }, {
        key: '_processOptions',
        value: function _processOptions(options) {
            options = (0, _assign3.default)({}, options);
            options.clip = (0, _assign3.default)({
                left: 0,
                right: 0,
                top: 0,
                bottom: 0
            }, options.clip);
            var _options2 = options,
                clip = _options2.clip,
                scale = _options2.scale,
                effects = _options2.effects;
            var top = clip.top,
                left = clip.left,
                bottom = clip.bottom,
                right = clip.right;

            options.effects = (0, _cloneDeep3.default)(effects);

            options.effects.forEach(function (effect) {
                effect.shadow.color = (0, _colorToRgba2.default)(effect.shadow.color);
                effect.stroke.color = (0, _colorToRgba2.default)(effect.stroke.color);
                effect.filling.color = (0, _colorToRgba2.default)(effect.filling.color);
                effect.filling.gradient.stops.forEach(function (stop) {
                    stop.color = (0, _colorToRgba2.default)(stop.color);
                });

                effect.shadow.blur = Math.ceil(effect.shadow.blur);
                effect.stroke.width = Math.ceil(effect.stroke.width);

                effect.expand.offset.left = left;
                effect.expand.offset.top = top;
                effect.expand.offset.right = right;
                effect.expand.offset.bottom = bottom;
                effect.expand.scale = scale;
            });

            return options;
        }
    }, {
        key: '_calcCanvasSize',
        value: function _calcCanvasSize(effects) {
            var keys = [['_offsetX', 'width'], ['_offsetY', 'height']];
            var edgeObjects = keys.map(function (_ref2) {
                var _ref3 = _slicedToArray(_ref2, 2),
                    offset = _ref3[0],
                    size = _ref3[1];

                return (0, _maxBy3.default)(effects, function (effect) {
                    return effect[offset] + effect[size];
                });
            });

            var _edgeObjects = _slicedToArray(edgeObjects, 2),
                rightObject = _edgeObjects[0],
                bottomObject = _edgeObjects[1];

            this.setCanvasSize({
                width: rightObject._offsetX + rightObject.width,
                height: bottomObject._offsetY + bottomObject.height
            });
        }
    }, {
        key: '_calcEffectsPosition',
        value: function _calcEffectsPosition(effects) {
            var _this3 = this;

            (0, _forEach3.default)(effects, function (effect) {
                var position = { x: 0, y: 0 };
                var effectOffset = { x: 0, y: 0 };
                var width = 0;
                var height = 0;

                effect.width = _this3._img.width;
                effect.height = _this3._img.height;
                effect._position = position;
                effect._effectOffset = effectOffset;

                if (!effect.enable) return;

                if (effect.offset.enable) {
                    position.x += effect.offset.x;
                    position.y += effect.offset.y;
                }
                if (effect.shadow.enable) {
                    var blur = effect.shadow.blur;

                    position.x -= blur - effect.shadow.offsetX;
                    position.y -= blur - effect.shadow.offsetY;

                    if (effect.shadow.offsetX <= 0) {
                        effectOffset.x += blur - effect.shadow.offsetX;
                        width += blur - effect.shadow.offsetX;
                    } else {
                        width += blur + effect.shadow.offsetX;
                    }

                    if (effect.shadow.offsetY <= 0) {
                        effectOffset.y += blur - effect.shadow.offsetY;
                        height += blur - effect.shadow.offsetY;
                    } else {
                        height += blur + effect.shadow.offsetY;
                    }
                }
                if (effect.stroke.enable) {
                    var strokeWidth = effect.stroke.width;

                    position.x -= strokeWidth;
                    position.y -= strokeWidth;
                    effectOffset.x = Math.max(effectOffset.x, strokeWidth);
                    effectOffset.y = Math.max(effectOffset.y, strokeWidth);
                    width = Math.max(width, effectOffset.x);
                    height = Math.max(height, effectOffset.y);
                }

                effect.width += width;
                effect.height += height;
            });

            var _map2 = ['x', 'y'].map(function (key) {
                return (0, _minBy3.default)(effects, '_position.' + key);
            }),
                _map3 = _slicedToArray(_map2, 2),
                leftObject = _map3[0],
                topObject = _map3[1];

            var globalOffset = {
                x: leftObject._effectOffset.x + Math.abs(leftObject.offset.x),
                y: topObject._effectOffset.y + Math.abs(topObject.offset.y)
            };

            effects.forEach(function (effect) {
                effect._offsetX = globalOffset.x;
                effect._offsetY = globalOffset.y;

                if (effect.enable && effect.offset.enable) {
                    effect._offsetX += effect.offset.x;
                    effect._offsetY += effect.offset.y;
                }
            });

            return globalOffset;
        }
    }]);

    return ImageEffect;
}();

ImageEffect.maskTypes = _model2.default.maskTypes;
ImageEffect.effects = _model2.default.effects;
ImageEffect.resizeThreshold = {
    width: 1000,
    height: 1000
};
exports.default = ImageEffect;
module.exports = exports['default'];