'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _assign2 = require('lodash/assign');

var _assign3 = _interopRequireDefault(_assign2);

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

exports.default = createExpandEffect;

var _fabric = require('fabric');

var _canvas = require('../../../canvas');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var floor = Math.floor,
    min = Math.min,
    abs = Math.abs;

var ExpandEffect = function () {
    function ExpandEffect(object, option) {
        _classCallCheck(this, ExpandEffect);

        this._defaultOption = {
            pixel: 1,
            top: 0,
            right: 0,
            left: 0,
            ratio: 1
        };

        var canvas = object.getElement();
        var width = canvas.width,
            height = canvas.height;


        this.$option = (0, _assign3.default)(this._defaultOption, option);
        this.$canvas = (0, _canvas.createCanvas)(width, height);

        var context = this.getContext();
        context.drawImage(canvas, 0, 0, width, height);
    }

    _createClass(ExpandEffect, [{
        key: 'getContext',
        value: function getContext() {
            return this.$canvas.getContext('2d');
        }
    }, {
        key: 'getExpandImageData',
        value: function getExpandImageData() {
            var type = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'top';
            var pixel = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1;
            var _$canvas = this.$canvas,
                width = _$canvas.width,
                height = _$canvas.height;
            var _$option = this.$option,
                top = _$option.top,
                right = _$option.right,
                bottom = _$option.bottom,
                left = _$option.left;

            var context = this.getContext();
            var positionMap = {
                top: [left, top, width - left - right, pixel],
                right: [width - right - 1, 0, pixel, height],
                bottom: [left, height - bottom - 1, width - left - right, pixel],
                left: [left, 0, pixel, height]
            };

            return context.getImageData.apply(context, positionMap[type]);
        }
    }, {
        key: 'addExpandEffect',
        value: function addExpandEffect() {
            var _this = this;

            ['top', 'bottom', 'left', 'right'].forEach(function (key) {
                _this.putExpandImageData(key);
            });
        }
    }, {
        key: 'putImageData',
        value: function putImageData(ctx, imageData, dx, dy, dirtyX, dirtyY, dirtyWidth, dirtyHeight) {
            ctx.putImageData(imageData, dx, dy, dirtyX, dirtyY, dirtyWidth, dirtyHeight);
        }
    }, {
        key: 'putExpandImageData',
        value: function putExpandImageData() {
            var _this2 = this;

            var type = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'top';
            var pixel = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1;

            var value = this.$option[type];

            if (value <= 0) {
                return;
            }

            var _$canvas2 = this.$canvas,
                width = _$canvas2.width,
                height = _$canvas2.height;
            var _$option2 = this.$option,
                _top = _$option2.top,
                _right = _$option2.right,
                _bottom = _$option2.bottom,
                _left = _$option2.left;

            var context = this.getContext();
            var imageData = this.getExpandImageData(type);
            var positionMap = {
                top: function top(i, imageData) {
                    return _this2.putImageData(context, imageData, _left, _top - i * pixel, 0, 0, width - _right - _left, pixel);
                },
                right: function right(i, imageData) {
                    return _this2.putImageData(context, imageData, width - _right + i * pixel, 0, 0, 0, pixel, height);
                },
                bottom: function bottom(i, imageData) {
                    return _this2.putImageData(context, imageData, _left, height - _bottom + i * pixel, 0, 0, width - _right - _left, pixel);
                },
                left: function left(i, imageData) {
                    return _this2.putImageData(context, imageData, _left - i * pixel, 0, 0, 0, pixel, height);
                }
            };

            var repeatTime = floor(value / pixel);
            var i = 0;

            while (i < repeatTime) {
                positionMap[type](i, imageData);
                i++;
            }
        }
    }, {
        key: 'getImageElement',
        value: function getImageElement() {
            var _$canvas3 = this.$canvas,
                width = _$canvas3.width,
                height = _$canvas3.height;


            return new _fabric.fabric.Image(this.$canvas, {
                width: width,
                height: height
            });
        }
    }]);

    return ExpandEffect;
}();

function createExpandEffect(object, options) {
    var top = options.top,
        left = options.left,
        bottom = options.bottom,
        right = options.right,
        _options$scale = options.scale,
        scale = _options$scale === undefined ? 1 : _options$scale;

    var absExpandValue = function absExpandValue(val) {
        return abs(min(floor(val / scale), 0));
    };
    var option = {
        top: absExpandValue(top),
        right: absExpandValue(right),
        left: absExpandValue(left),
        bottom: absExpandValue(bottom)
    };
    var obj = new ExpandEffect(object, option);

    obj.addExpandEffect();

    return obj.getImageElement();
};
module.exports = exports['default'];