'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = createGradient;

var _fabric = require('fabric');

var _getGradientPoint = require('../get-gradient-point');

var _getGradientPoint2 = _interopRequireDefault(_getGradientPoint);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createGradient(object) {
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    var width = object.width * object.scaleX;
    var height = object.height * object.scaleY;

    var gradient = new _fabric.fabric.Rect({
        width: width,
        height: height,
        left: object.left,
        top: object.top
    });

    var angle = options.angle === 0 ? options.angle : 180 - options.angle;
    var quadrant = Math.max(Math.abs(Math.ceil(angle / 90)), 1);
    var radius = Math.sqrt(Math.pow(width, 2) + Math.pow(height, 2));
    var point = (0, _getGradientPoint2.default)(angle, radius);
    var x1 = [1, 4].includes(quadrant) ? 0 : width;
    var y1 = [1, 2].includes(quadrant) ? 0 : height;
    var x2 = Math.max(x1 + (point.x2 - point.x1), 0);
    var y2 = Math.max(y1 + (point.y1 - point.y2), 0);

    var gradientDefaultOptions = {
        linear: {
            type: 'linear',
            x1: x1,
            x2: x2,
            y1: y1,
            y2: y2,
            colorStops: {
                0: 'white',
                1: 'black'
            }
        },
        radial: {
            type: 'radial',
            r1: width / 2 - 10,
            r2: width / 2,
            x1: width / 2,
            y1: height / 2,
            x2: width / 2,
            y2: height / 2,
            colorStops: {
                0: 'white',
                1: 'transparent'
            }
        }
    };
    var gradientOptions = Object.assign({}, gradientDefaultOptions[options.gradientType || 'linear'], options);

    gradient.setGradient('fill', gradientOptions);

    return gradient;
}
module.exports = exports['default'];