'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _jquery = require('jquery');

var _jquery2 = _interopRequireDefault(_jquery);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _fontfacePolyfill = require('./fontface-polyfill');

var _fontfacePolyfill2 = _interopRequireDefault(_fontfacePolyfill);

var _readFile = require('./read-file');

var _readFile2 = _interopRequireDefault(_readFile);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var caches = {
    image: {},
    font: {},
    xml: {}
};

var crossOriginParam = 'cross_origin=1';

var isCrossOriginUrl = function isCrossOriginUrl(url) {
    var origin = location.host;

    return url.indexOf('data:') !== 0 && url.indexOf(origin) < 0;
};

var getComputedUrl = function getComputedUrl() {
    var url = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
    var fitCrossOrigin = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

    if (!url || !fitCrossOrigin) {
        return url;
    }

    if (isCrossOriginUrl(url) && url.indexOf(crossOriginParam) < 0) {
        url += url.indexOf('?') < 0 ? '?' : '&';
        url += crossOriginParam;
    }

    return url;
};

var _loadImage = function _loadImage(url) {
    return new _bluebird2.default(function (resolve, reject) {
        var img = new Image();

        if (isCrossOriginUrl(url)) {
            img.crossOrigin = 'Anonymous';
        }

        img.onload = function () {
            resolve(img);
        };

        img.onerror = function () {
            var msg = 'Image load error: ' + url;

            reject(new Error(msg));
        };

        img.src = url;
    });
};

exports.default = {
    isCrossOriginUrl: isCrossOriginUrl,
    getComputedUrl: getComputedUrl,
    _loadImage: _loadImage,
    loadImage: function loadImage(url) {
        var fitCrossOrigin = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

        var imgCaches = caches.image;

        url = getComputedUrl(url, fitCrossOrigin);

        if (!imgCaches[url]) {
            imgCaches[url] = _loadImage(url);
        }

        return imgCaches[url];
    },
    loadXML: function loadXML(url) {
        var xmlCaches = caches.xml;

        if (!xmlCaches[url]) {
            xmlCaches[url] = _bluebird2.default.try(function () {
                return _jquery2.default.get(url, _jquery2.default.noop, 'xml');
            }).catch(function (err) {
                err.message = 'Svg load error: ' + err.message;

                throw err;
            });
        }

        return xmlCaches[url];
    },
    loadFont: function loadFont(options) {
        var timeout = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 10000;

        var fontCaches = caches.font;
        var name = options.name;

        if (!fontCaches[name]) {
            var family = options.family || name;
            var urls = ['local("' + family + '")'];

            if (name !== family) {
                urls.push('local("' + name + '")');
            }
            if (options.woff || options.url) {
                urls.push('url("' + (options.woff || options.url) + '")');
            }
            if (options.ttf) {
                urls.push('url("' + options.ttf + '")');
            }

            var font = new _fontfacePolyfill2.default(family, urls.join(','), {
                weight: options.weight || 'normal',
                style: options.style || 'normal',
                display: options.display || 'auto'
            });

            document.fonts.add(font);

            fontCaches[name] = _bluebird2.default.try(function () {
                return font.load();
            }).timeout(timeout, 'Font [' + family + '] load timeout(' + timeout + 'ms)').catch(function (err) {
                delete fontCaches[name];

                throw err;
            });
        }

        return fontCaches[name];
    },


    readFile: _readFile2.default
};
module.exports = exports['default'];