'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _trim2 = require('lodash/trim');

var _trim3 = _interopRequireDefault(_trim2);

var _cloneDeep2 = require('lodash/cloneDeep');

var _cloneDeep3 = _interopRequireDefault(_cloneDeep2);

var _isObject2 = require('lodash/isObject');

var _isObject3 = _interopRequireDefault(_isObject2);

var _unescape2 = require('lodash/unescape');

var _unescape3 = _interopRequireDefault(_unescape2);

var _escape2 = require('lodash/escape');

var _escape3 = _interopRequireDefault(_escape2);

var _isArray2 = require('lodash/isArray');

var _isArray3 = _interopRequireDefault(_isArray2);

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _config = require('../config');

var _jquery = require('jquery');

var _jquery2 = _interopRequireDefault(_jquery);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var rTwoSpace = /\s\s/g;
var rBr = /<br\s*\/?>/ig;
var rBreakLine = /\r?\n/gm;

var rVoidWidth = /\u200b/gm;

var removeLastBr = function removeLastBr(elem) {
    var lastChild = elem.lastChild;
    var blockTags = ['P', 'DIV'];
    var next = elem.nextSibling;

    if (!blockTags.includes(elem.nodeName) && (!next || !['BBR'].concat(blockTags).includes(next.nodeName))) {
        return;
    }

    while (lastChild) {
        if (lastChild.nodeName === 'BR') {
            lastChild.parentNode.removeChild(lastChild);
            break;
        } else {
            lastChild = lastChild.lastChild;
        }
    }

    return null;
};

exports.default = {
    fromHTML: function fromHTML(contentModel) {
        var _this = this;

        var zoom = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1;

        if (!(0, _isArray3.default)(contentModel)) throw new TypeError('contentModel must be a array');

        var result = '';
        contentModel.forEach(function (node) {
            if (node && node.content) {
                var style = {};

                Object.keys(node).forEach(function (name) {
                    if (_config.styleMap[name]) {
                        style[name] = name === 'fontSize' ? node[name] * zoom : node[name];
                    }
                });

                style = _this._getStyleString(style);

                var content = node.content.replace(rBr, '\n');

                content = (0, _escape3.default)((0, _unescape3.default)(content)).replace(rBreakLine, '<br>').replace(rTwoSpace, '&nbsp; ');

                result += !style ? content : '<span ' + _config.CUSTOMIZE_MARK + '="1" style=\'' + style + '\'>' + content + '</span>';
            }
        });
        return result;
    },
    fromJSON: function fromJSON(html) {
        var inheritStyle = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
        var zoom = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 1;

        if (!html) return [];

        var _inheritStyle = {};
        var $elem = (0, _jquery2.default)('<span></span>').append(html);
        var elem = $elem[0];
        var blocks = ['P', 'DIV'];

        _jquery2.default.each($elem.find('*').not('br').get().reverse(), function (i, el) {
            var $el = (0, _jquery2.default)(el);
            var html = $el.html().trim();

            if (html === '') {
                $el.remove();
            } else if (html === '<br>') {
                $el.replaceWith('<br>');
            }
        });

        $elem.find('p,div').each(function (i, el) {
            var next = el.nextSibling;
            var prev = el.previousSibling;

            if (prev && prev.nodeName !== 'BR') {
                var br = (0, _jquery2.default)('<bbr>');
                (0, _jquery2.default)(el).before(br);
            }

            if (next && !blocks.includes(next.nodeName)) {
                (0, _jquery2.default)(el).after('<br>');
            }
        });

        for (var k in inheritStyle) {
            if (_config.styleMap[k]) _inheritStyle[k] = inheritStyle[k];
        }

        var children = this._getChildJSON(elem, _inheritStyle);

        children.forEach(function (item) {
            if (item.fontSize) item.fontSize = item.fontSize / zoom;
        });

        return this.mergeChild(children, true);
    },
    injectStyleProps: function injectStyleProps(contentModel) {
        var injectStyle = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

        if (!(0, _isArray3.default)(contentModel)) throw new TypeError('contentModel must be a array');
        if (!(0, _isObject3.default)(injectStyle)) throw new TypeError('injectStyle must be a object');

        var newModel = (0, _cloneDeep3.default)(contentModel);
        newModel.forEach(function (item) {
            for (var k in injectStyle) {
                if (_config.styleMap[k]) {
                    item[k] = injectStyle[k];
                }
            }
        });

        return newModel;
    },
    removeStyleProps: function removeStyleProps(contentModel) {
        var removeStyle = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

        if (!(0, _isArray3.default)(contentModel)) throw new TypeError('contentModel must be a array');
        if (!(0, _isObject3.default)(removeStyle)) throw new TypeError('removeStyle must be a object');

        var newModel = (0, _cloneDeep3.default)(contentModel);

        newModel.forEach(function (item) {
            for (var k in removeStyle) {
                if (_config.styleMap[k]) {
                    delete item[k];
                }
            }
        });

        return this.mergeChild(newModel);
    },
    getAllEqualStyleProps: function getAllEqualStyleProps(contentModel) {
        if (!(0, _isArray3.default)(contentModel)) throw new TypeError('contentModel must be a array');

        var props = {};
        var defaultValueMap = {
            textDecoration: 'none',
            fontStyle: 'normal',
            fontWeight: 400
        };

        for (var k in _config.styleMap) {
            if (k.indexOf('-') === -1) props[k] = null;
        }

        var _loop = function _loop(_k) {
            contentModel.forEach(function (item) {
                var val = item[_k] || defaultValueMap[_k];

                if (props[_k] === null) {
                    props[_k] = val;
                } else if (props[_k] !== val) {
                    delete props[_k];
                }
            });
        };

        for (var _k in props) {
            _loop(_k);
        }

        for (var _k2 in props) {
            if (props[_k2] === null || props[_k2] === undefined) delete props[_k2];
        }

        return props;
    },
    mergeChild: function mergeChild(contentModel, notDeepCopy) {
        var _this2 = this;

        if (!(0, _isArray3.default)(contentModel)) throw new TypeError('contentModel must be a array');

        var br = '';
        var newModel = notDeepCopy ? contentModel : (0, _cloneDeep3.default)(contentModel);
        var result = newModel.reduce(function (newChildren, next) {
            var lastIndex = newChildren.length - 1;
            var lastChild = newChildren[lastIndex];

            if (next.tag === 'br') {
                br += '<br>';
                return newChildren;
            }

            next.content = br + next.content;
            br = '';

            if (!lastChild) {
                newChildren.push(next);
                return newChildren;
            }

            var styleEqual = _this2._getStyleString(lastChild) === _this2._getStyleString(next);

            if (styleEqual) {
                lastChild.content += next.content;
            } else {
                newChildren.push(next);
            }

            return newChildren;
        }, []);

        if (result[result.length - 1]) {
            result[result.length - 1].content += br;
        } else if (br) {
            result = [{
                content: br
            }];
        }

        return result;
    },
    _getChildJSON: function _getChildJSON(elem) {
        var _this3 = this;

        var inheritStyle = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
        var limit = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

        if (!elem.textContent && limit) return [];

        var children = Array.prototype.slice.call(elem.childNodes);
        var result = [];
        var nodeType = void 0;
        var nodeTag = void 0;
        var elemResult = void 0;

        var nodeType1 = function nodeType1(curElem) {
            var styleStr = curElem.getAttribute('style') || '';
            var styleJSON = styleStr ? _this3._getStyleJSON(styleStr) : {};
            var tagToStyle = _config.tagMap[nodeTag];

            if (nodeTag === 'br' || nodeTag === 'bbr') {
                elemResult = { tag: 'br' };
            } else {
                removeLastBr(curElem);

                tagToStyle && Object.assign(styleJSON, typeof tagToStyle === 'function' ? tagToStyle(inheritStyle, curElem) : tagToStyle);

                result = result.concat(_this3._getChildJSON(curElem, _extends({}, inheritStyle, styleJSON)));
            }
        };

        var nodeType3 = function nodeType3(curElem) {
            if (curElem.textContent) {
                elemResult = _extends({}, inheritStyle, {
                    content: (0, _escape3.default)(curElem.textContent).replace(rVoidWidth, '')
                });
            }
        };

        children.forEach(function (curElem) {
            nodeType = curElem.nodeType;
            nodeTag = curElem.nodeName.toLowerCase().replace('#', '');
            elemResult = null;

            if (nodeType === 1) {
                nodeType1(curElem);
            } else if (nodeType === 3) {
                    nodeType3(curElem);
                }

            elemResult && result.push(elemResult);
        });
        return result;
    },
    _getStyleJSON: function _getStyleJSON(style) {
        var result = {};

        style.split(';').forEach(function (expression) {
            var _expression$split = expression.split(':'),
                _expression$split2 = _slicedToArray(_expression$split, 2),
                name = _expression$split2[0],
                value = _expression$split2[1];

            name = _config.styleMap[(0, _trim3.default)(name)];
            value = (0, _trim3.default)(value);

            if (name) result[name] = _config.styleValueMap[name] ? _config.styleValueMap[name](value) : value;
        });

        return result;
    },
    _getStyleString: function _getStyleString(style) {
        var result = '';

        Object.keys(style).forEach(function (name) {
            var value = style[name];
            name = _config.styleMap[name];

            if (name) result += name + ': ' + (_config.styleValueMap[name] ? _config.styleValueMap[name](value) : value) + ';';
        });
        return result;
    }
};
module.exports = exports['default'];