'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _pick2 = require('lodash/pick');

var _pick3 = _interopRequireDefault(_pick2);

var _matrix = require('./matrix');

var _matrix2 = _interopRequireDefault(_matrix);

var _point = require('./point');

var _point2 = _interopRequireDefault(_point);

var _observablePoint = require('./observable-point');

var _observablePoint2 = _interopRequireDefault(_observablePoint);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function Transform() {
    this._worldID = 0;
    this.localTransform = new _matrix2.default();

    this.position = new _point2.default(0.0);

    this.scale = new _point2.default(1, 1);

    this.skew = new _observablePoint2.default(this.updateSkew, this, 0, 0);

    this.pivot = new _point2.default(0.0);

    this._rotation = 0;

    this._sr = Math.sin(0);
    this._cr = Math.cos(0);
    this._cy = Math.cos(0);
    this._sy = Math.sin(0);
    this._nsx = Math.sin(0);
    this._cx = Math.cos(0);
}

Transform.prototype.updateSkew = function () {
    this._nsx = Math.tan(this.skew.x);
    this._sy = Math.tan(this.skew.y);
};

Transform.prototype.updateLocalTransform = function () {
    var lt = this.localTransform;
    var a = void 0,
        b = void 0,
        c = void 0,
        d = void 0;

    a = this._cr * this.scale.x;
    b = this._sr * this.scale.x;
    c = -this._sr * this.scale.y;
    d = this._cr * this.scale.y;

    lt.a = this._cy * a + this._sy * c;
    lt.b = this._cy * b + this._sy * d;
    lt.c = this._nsx * a + this._cx * c;
    lt.d = this._nsx * b + this._cx * d;

    lt.tx = this.position.x;
    lt.ty = this.position.y;
};

Transform.prototype.updateTransform = function () {};

Transform.prototype.setFromMatrix = function (matrix) {
    matrix.decompose(this);
};

Object.defineProperties(Transform.prototype, {
    rotation: {
        get: function get() {
            return this._rotation;
        },
        set: function set(value) {
            this._rotation = value;
            this._sr = Math.sin(value);
            this._cr = Math.cos(value);
        }
    }
});

Transform.prototype.toJSON = function () {
    this.updateLocalTransform();

    return (0, _pick3.default)(this.localTransform, ['a', 'b', 'c', 'd', 'tx', 'ty']);
};

Transform.prototype.toArray = function () {
    this.updateLocalTransform();

    var localTransform = this.localTransform;

    return [localTransform.a, localTransform.b, localTransform.c, localTransform.d, localTransform.tx, localTransform.ty];
};

Transform.prototype.toString = function () {
    var rZeroEnd = /\.?0+$/;
    var arr = this.toArray();

    arr = arr.map(function (n) {
        n = n.toFixed(20);

        return n.replace(rZeroEnd, '');
    });

    return 'matrix(' + arr.join(', ') + ')';
};

exports.default = Transform;
module.exports = exports['default'];