'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _debounce2 = require('lodash/debounce');

var _debounce3 = _interopRequireDefault(_debounce2);

var _utils = require('../utils/utils');

var _utils2 = _interopRequireDefault(_utils);

var _editorColorPicker = require('./editor-color-picker.html');

var _editorColorPicker2 = _interopRequireDefault(_editorColorPicker);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = {
    template: _editorColorPicker2.default,
    props: {
        'change': Function,
        'color': {
            type: String,
            default: '#000000'
        },
        'active': {
            type: Boolean,
            default: false
        },
        'param': {}
    },
    data: function data() {
        return {
            isShow: this.active,

            pickerType: 0 };
    },


    computed: {
        pickerColor: {
            get: function get() {
                return this.color;
            },
            set: function set(value) {
                this.$emit('update:color', value);

                if (this.change) {
                    this.change(value, this.param);
                }
            }
        },

        isLightColor: function isLightColor() {
            if (!this.pickerColor) {
                return false;
            }

            var hex = this.parseHex(this.pickerColor).slice(1);
            var rgb = hex.match(/\w{2}/g) || [];
            var max = 224;

            if (parseInt(rgb[0], 16) > max && parseInt(rgb[1], 16) > max && parseInt(rgb[2], 16) > max) {
                return true;
            }

            return false;
        }
    },

    methods: {
        parseHex: function parseHex(str) {
            str = String(str).replace(/[^A-F0-9]/ig, '');

            if (str.length === 3) {
                str = str[0] + str[0] + str[1] + str[1] + str[2] + str[2];
            }

            return '#' + str;
        },
        toggleShow: function toggleShow() {
            this.isShow = !this.isShow;
        },
        initOutClick: function initOutClick() {
            var self = this;
            var offOutClick = _utils2.default.addOutClick({
                element: this.$el,
                callback: function callback() {
                    self.isShow = false;
                }
            });

            this.$on('destroy', offOutClick);
        },

        addHistory: (0, _debounce3.default)(function () {
            if (this.isShow && !this._isDestroyed) {
                this.$events.$emit('color.history.add', this.pickerColor);
            }
        }, 1000),

        setColor: function setColor(color, isChange) {
            this.$emit('update:color', color);

            if (isChange) {
                this.$events.$emit('hue.color.change', color);
            }
        },
        switchPickerType: function switchPickerType() {
            this.pickerType = this.pickerType === 0 ? 1 : 0;
        }
    },

    watch: {
        pickerColor: function pickerColor(color) {
            this.$emit('update:color', color);

            if (this.change) {
                this.change(color, this.param);
            }

            this.addHistory();
        }
    },

    mounted: function mounted() {
        this.initOutClick();
    },
    beforeDestroy: function beforeDestroy() {
        this.$events.$emit('color.history.add', this.pickerColor);
        this.$emit('destroy');
    }
};
module.exports = exports['default'];