'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _jquery = require('jquery');

var _jquery2 = _interopRequireDefault(_jquery);

var _utils = require('../utils/utils');

var _utils2 = _interopRequireDefault(_utils);

var _editorOpacityPicker = require('./editor-opacity-picker.html');

var _editorOpacityPicker2 = _interopRequireDefault(_editorOpacityPicker);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = {
    template: _editorOpacityPicker2.default,
    props: ['value', 'change', 'inline'],
    data: function data() {
        return {
            active: false,
            opacity: 1
        };
    },

    computed: {
        opacityPercent: {
            get: function get() {
                var num = 100 * this.opacity;

                num = Math.round(num);

                return num + '%';
            },
            set: function set(num) {
                var val = parseFloat(num) || 0;

                this.opacity = val / 100;
            }
        }
    },
    methods: {
        toggleActive: function toggleActive(e) {
            e.preventDefault();

            if (this.active && e.target.nodeName === 'INPUT') {
                return;
            }

            this.active = !this.active;
        },
        onKeydown: function onKeydown(e) {
            var ratio = 20;
            var offset = 39 - e.keyCode;

            if (Math.abs(offset) === 1) {
                var val = this.opacity + offset / ratio;

                this.set(val);
            }
        },
        set: function set(val) {
            val = Math.max(0, Math.min(1, val));

            this.opacity = val;
        },
        initInputFocus: function initInputFocus() {
            var element = this.$el;

            this.$watch('active', function () {
                this.$nextTick(function () {
                    var inp = (0, _jquery2.default)('input', element).eq(0);
                    var active = this.active;

                    if (active) {
                        inp.focus().select();
                    } else {
                        inp.blur();
                    }
                });
            });
        },
        initOutClick: function initOutClick() {
            var self = this;
            var offOutClick = _utils2.default.addOutClick({
                element: this.$el,
                callback: function callback() {
                    self.active = false;
                }
            });

            this.$on('destroy', offOutClick);
        }
    },

    mounted: function mounted() {
        if (this.inline) {
            this.active = true;
        } else {
            this.initOutClick();
        }

        this.opacity = this.value;

        this.$watch('value', function (a) {
            this.opacity = a;
        });

        this.$watch('opacity', function () {
            if (this.opacity === this.value) {
                return;
            }

            if (!this.change) {
                this.value = this.value;
            } else {
                this.change(this.opacity);
            }
        });

        this.initInputFocus();
    },
    beforeDestroy: function beforeDestroy() {
        this.$emit('destroy');
    }
};
module.exports = exports['default'];