'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _jquery = require('jquery');

var _jquery2 = _interopRequireDefault(_jquery);

var _utils = require('../utils/utils');

var _utils2 = _interopRequireDefault(_utils);

var _editorSelectPicker = require('./editor-select-picker.html');

var _editorSelectPicker2 = _interopRequireDefault(_editorSelectPicker);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = {
    template: _editorSelectPicker2.default,
    props: ['change', 'value', 'items', 'is-equal'],
    data: function data() {
        return {
            active: false,
            currItem: null,
            pushUp: false,
            listMaxHeight: ''
        };
    },

    methods: {
        toggleActive: function toggleActive() {
            this.active = !this.active;
            this.listMaxHeight = '';
            this.pushUp = false;

            if (this.active) {
                this.$nextTick(function () {
                    this.updateListPosition();

                    this.$nextTick(this.scrollToCurrent);
                });
            }
        },
        syncCurrItem: function syncCurrItem() {
            var _this = this;

            var currItem = this.items.find(function (item) {
                if (_this.isEqual(item)) {
                    return true;
                }

                return false;
            });

            this.currItem = currItem || null;
        },
        set: function set(item) {
            var _this2 = this;

            this.currItem = item || null;

            this.$nextTick(function () {
                _this2.active = false;
            });
        },
        initOutClick: function initOutClick() {
            var self = this;
            var offOutClick = _utils2.default.addOutClick({
                element: this.$el,
                callback: function callback() {
                    self.active = false;
                }
            });

            this.$on('destroy', offOutClick);
        },
        updateListPosition: function updateListPosition() {
            var elementRect = this.$refs.list.getBoundingClientRect();
            var offset = document.documentElement.offsetHeight - elementRect.bottom;

            var needResize = offset < 20;

            if (needResize) {
                var maxHeight = elementRect.height + offset - 20;
                if (maxHeight < 150) {
                    this.pushUp = true;
                    this.maxHeight = '';
                } else {
                    this.pushUp = false;
                    this.listMaxHeight = maxHeight + 'px';
                }
            } else {
                    this.pushUp = false;
                    this.listMaxHeight = '';
                }
        },
        scrollToCurrent: function scrollToCurrent() {
            var list = this.$refs.list;
            var currentEle = (0, _jquery2.default)(list).find('li.current')[0];
            if (currentEle) {
                currentEle.offsetParent.scrollTop = currentEle.offsetTop;
            }
        }
    },
    mounted: function mounted() {
        var _this3 = this;

        var vm = this;

        if (!this.isEqual) {
            this.isEqual = function (item) {
                return item === vm.currItem;
            };
        }

        this.syncCurrItem();

        this.$watch('value', function () {
            _this3.syncCurrItem();
        });

        this.$watch('currItem', function (currItem) {
            if (!_this3.active || currItem === _this3.value) {
                return;
            }

            if (!_this3.change) {
                _this3.value = _this3.currItem;
            } else {
                _this3.change(_this3.currItem);
            }
        });

        this.initOutClick();
    },
    beforeDestroy: function beforeDestroy() {
        this.$emit('destroy');
    }
};
module.exports = exports['default'];